@@ -1,10 +1,15 @@
+// SPDX-License-Identifier: MIT
+//
+// Copyright 2016-2025, Johann Tuffe.
+
 //! A module to parse Open Document Spreadsheets
 //!
-//! # Reference
-//! OASIS Open Document Format for Office Application 1.2 (ODF 1.2)
-//! http://docs.oasis-open.org/office/v1.2/OpenDocument-v1.2.pdf
-
-use std::borrow::Cow;
+/// # Reference
+///
+/// OASIS Open Document Format for Office Application 1.2 ([ODF 1.2]).
+///
+/// [ODF 1.2]: http://docs.oasis-open.org/office/v1.2/OpenDocument-v1.2.pdf
+///
 use std::collections::{BTreeMap, HashMap};
 use std::io::{BufReader, Read, Seek};
 
@@ -15,6 +20,7 @@ use quick_xml::Reader as XmlReader;
 use zip::read::{ZipArchive, ZipFile};
 use zip::result::ZipError;
 
+use crate::utils::unescape_entity_to_buffer;
 use crate::vba::VbaProject;
 use crate::{Data, DataType, HeaderRow, Metadata, Range, Reader, Sheet, SheetType, SheetVisible};
 use std::marker::PhantomData;
@@ -77,9 +83,10 @@ struct OdsOptions {
 from_err!(std::io::Error, OdsError, Io);
 from_err!(zip::result::ZipError, OdsError, Zip);
 from_err!(quick_xml::Error, OdsError, Xml);
-from_err!(std::string::ParseError, OdsError, Parse);
+from_err!(std::str::ParseBoolError, OdsError, ParseBool);
 from_err!(std::num::ParseFloatError, OdsError, ParseFloat);
-from_err!(quick_xml::events::attributes::AttrError, OdsError, Xml);
+from_err!(std::num::ParseIntError, OdsError, ParseInt);
+from_err!(quick_xml::events::attributes::AttrError, OdsError, XmlAttr);
 from_err!(quick_xml::encoding::EncodingError, OdsError, Xml);
 
 impl std::fmt::Display for OdsError {
@@ -123,11 +130,14 @@ impl std::error::Error for OdsError {
     }
 }
 
-/// An OpenDocument Spreadsheet document parser
+/// An `OpenDocument` Spreadsheet document parser
 ///
 /// # Reference
-/// OASIS Open Document Format for Office Application 1.2 (ODF 1.2)
-/// http://docs.oasis-open.org/office/v1.2/OpenDocument-v1.2.pdf
+///
+/// OASIS Open Document Format for Office Application 1.2 ([ODF 1.2]).
+///
+/// [ODF 1.2]: http://docs.oasis-open.org/office/v1.2/OpenDocument-v1.2.pdf
+///
 pub struct Ods<RS> {
     sheets: BTreeMap<String, (Range<Data>, Range<String>)>,
     metadata: Metadata,
@@ -191,8 +201,8 @@ where
     }
 
     /// Gets `VbaProject`
-    fn vba_project(&mut self) -> Option<Result<Cow<'_, VbaProject>, OdsError>> {
-        None
+    fn vba_project(&mut self) -> Result<Option<VbaProject>, OdsError> {
+        Ok(None)
     }
 
     /// Read sheets from workbook.xml and get their corresponding path from relationships
@@ -269,12 +279,10 @@ fn check_for_password_protected<RS: Read + Seek>(zip: &mut ZipArchive<RS>) -> Re
     let mut inner = Vec::new();
     loop {
         match reader.read_event_into(&mut buf) {
-            Ok(Event::Start(ref e)) if e.name() == QName(b"manifest:file-entry") => {
+            Ok(Event::Start(e)) if e.name() == QName(b"manifest:file-entry") => {
                 loop {
                     match reader.read_event_into(&mut inner) {
-                        Ok(Event::Start(ref e))
-                            if e.name() == QName(b"manifest:encryption-data") =>
-                        {
+                        Ok(Event::Start(e)) if e.name() == QName(b"manifest:encryption-data") => {
                             return Err(OdsError::Password)
                         }
                         Ok(Event::Eof) => break,
@@ -282,13 +290,13 @@ fn check_for_password_protected<RS: Read + Seek>(zip: &mut ZipArchive<RS>) -> Re
                         _ => (),
                     }
                 }
-                inner.clear()
+                inner.clear();
             }
             Ok(Event::Eof) => break,
             Err(e) => return Err(OdsError::Xml(e)),
             _ => (),
         }
-        buf.clear()
+        buf.clear();
     }
 
     Ok(())
@@ -317,51 +325,44 @@ fn parse_content<RS: Read + Seek>(mut zip: ZipArchive<RS>) -> Result<Content, Od
     let mut style_name: Option<String> = None;
     loop {
         match reader.read_event_into(&mut buf) {
-            Ok(Event::Start(ref e)) if e.name() == QName(b"style:style") => {
+            Ok(Event::Start(e)) if e.name() == QName(b"style:style") => {
                 style_name = e
                     .try_get_attribute(b"style:name")?
                     .map(|a| a.decode_and_unescape_value(reader.decoder()))
-                    .transpose()
-                    .map_err(OdsError::Xml)?
-                    .map(|x| x.to_string())
+                    .transpose()?
+                    .map(|x| x.to_string());
             }
-            Ok(Event::Start(ref e))
+            Ok(Event::Start(e))
                 if style_name.is_some() && e.name() == QName(b"style:table-properties") =>
             {
                 let visible = match e.try_get_attribute(b"table:display")? {
-                    Some(a) => match a
-                        .decode_and_unescape_value(reader.decoder())
-                        .map_err(OdsError::Xml)?
-                        .parse()
-                        .map_err(OdsError::ParseBool)?
-                    {
-                        true => SheetVisible::Visible,
-                        false => SheetVisible::Hidden,
-                    },
+                    Some(a) => {
+                        if a.decode_and_unescape_value(reader.decoder())?.parse()? {
+                            SheetVisible::Visible
+                        } else {
+                            SheetVisible::Hidden
+                        }
+                    }
                     None => SheetVisible::Visible,
                 };
                 styles.insert(style_name.clone(), visible);
             }
-            Ok(Event::Start(ref e)) if e.name() == QName(b"table:table") => {
+            Ok(Event::Start(e)) if e.name() == QName(b"table:table") => {
                 let visible = styles
                     .get(
                         &e.try_get_attribute(b"table:style-name")?
                             .map(|a| a.decode_and_unescape_value(reader.decoder()))
-                            .transpose()
-                            .map_err(OdsError::Xml)?
+                            .transpose()?
                             .map(|x| x.to_string()),
                     )
                     .cloned()
                     .unwrap_or(SheetVisible::Visible);
-                if let Some(ref a) = e
+                if let Some(a) = e
                     .attributes()
                     .filter_map(|a| a.ok())
                     .find(|a| a.key == QName(b"table:name"))
                 {
-                    let name = a
-                        .decode_and_unescape_value(reader.decoder())
-                        .map_err(OdsError::Xml)?
-                        .to_string();
+                    let name = a.decode_and_unescape_value(reader.decoder())?.to_string();
                     let (range, formulas) = read_table(&mut reader)?;
                     sheets_metadata.push(Sheet {
                         name: name.clone(),
@@ -371,7 +372,7 @@ fn parse_content<RS: Read + Seek>(mut zip: ZipArchive<RS>) -> Result<Content, Od
                     sheets.insert(name, (range, formulas));
                 }
             }
-            Ok(Event::Start(ref e)) if e.name() == QName(b"table:named-expressions") => {
+            Ok(Event::Start(e)) if e.name() == QName(b"table:named-expressions") => {
                 defined_names = read_named_expressions(&mut reader)?;
             }
             Ok(Event::Eof) => break,
@@ -401,13 +402,9 @@ where
     cols.push(0);
     loop {
         match reader.read_event_into(&mut buf) {
-            Ok(Event::Start(ref e)) if e.name() == QName(b"table:table-row") => {
+            Ok(Event::Start(e)) if e.name() == QName(b"table:table-row") => {
                 let row_repeats = match e.try_get_attribute(b"table:number-rows-repeated")? {
-                    Some(c) => c
-                        .decode_and_unescape_value(reader.decoder())
-                        .map_err(OdsError::Xml)?
-                        .parse()
-                        .map_err(OdsError::ParseInt)?,
+                    Some(c) => c.decode_and_unescape_value(reader.decoder())?.parse()?,
                     None => 1,
                 };
                 read_row(
@@ -420,7 +417,7 @@ where
                 cols.push(cells.len());
                 rows_repeats.push(row_repeats);
             }
-            Ok(Event::End(ref e)) if e.name() == QName(b"table:table") => break,
+            Ok(Event::End(e)) if e.name() == QName(b"table:table") => break,
             Err(e) => return Err(OdsError::Xml(e)),
             Ok(_) => (),
         }
@@ -468,9 +465,8 @@ fn get_range<T: Default + Clone + PartialEq>(
             }
         }
     }
-    let row_min = match row_min {
-        Some(min) => min,
-        _ => return Range::default(),
+    let Some(row_min) = row_min else {
+        return Range::default();
     };
 
     // rebuild cells into its smallest non empty area
@@ -502,11 +498,11 @@ fn get_range<T: Default + Clone + PartialEq>(
                 }
                 empty_row_repeats = 0;
                 consecutive_empty_rows = 0;
-            };
+            }
 
             if row_repeats > 1 {
                 row_max = row_max + row_repeats - 1;
-            };
+            }
 
             for _ in 0..row_repeats {
                 match row.len().cmp(&(col_max + 1)) {
@@ -548,19 +544,15 @@ where
     loop {
         row_buf.clear();
         match reader.read_event_into(row_buf) {
-            Ok(Event::Start(ref e))
+            Ok(Event::Start(e))
                 if e.name() == QName(b"table:table-cell")
                     || e.name() == QName(b"table:covered-table-cell") =>
             {
                 let mut repeats = 1;
                 for a in e.attributes() {
-                    let a = a.map_err(OdsError::XmlAttr)?;
+                    let a = a?;
                     if a.key == QName(b"table:number-columns-repeated") {
-                        repeats = reader
-                            .decoder()
-                            .decode(&a.value)?
-                            .parse()
-                            .map_err(OdsError::ParseInt)?;
+                        repeats = reader.decoder().decode(&a.value)?.parse()?;
                         break;
                     }
                 }
@@ -585,12 +577,12 @@ where
                     reader.read_to_end_into(e.name(), cell_buf)?;
                 }
             }
-            Ok(Event::End(ref e)) if e.name() == QName(b"table:table-row") => break,
+            Ok(Event::End(e)) if e.name() == QName(b"table:table-row") => break,
             Err(e) => return Err(OdsError::Xml(e)),
             Ok(e) => {
                 return Err(OdsError::Mismatch {
                     expected: "table-cell",
-                    found: format!("{:?}", e),
+                    found: format!("{e:?}"),
                 });
             }
         }
@@ -614,20 +606,17 @@ where
     let mut val = Data::Empty;
     let mut formula = String::new();
     for a in atts {
-        let a = a.map_err(OdsError::XmlAttr)?;
+        let a = a?;
         match a.key {
             QName(b"office:value") if !is_value_set => {
                 let v = reader.decoder().decode(&a.value)?;
-                val = Data::Float(v.parse().map_err(OdsError::ParseFloat)?);
+                val = Data::Float(v.parse()?);
                 is_value_set = true;
             }
             QName(b"office:string-value" | b"office:date-value" | b"office:time-value")
                 if !is_value_set =>
             {
-                let attr = a
-                    .decode_and_unescape_value(reader.decoder())
-                    .map_err(OdsError::Xml)?
-                    .to_string();
+                let attr = a.decode_and_unescape_value(reader.decoder())?.to_string();
                 val = match a.key {
                     QName(b"office:date-value") => Data::DateTimeIso(attr),
                     QName(b"office:time-value") => Data::DurationIso(attr),
@@ -642,10 +631,7 @@ where
             }
             QName(b"office:value-type") if !is_value_set => is_string = &*a.value == b"string",
             QName(b"table:formula") => {
-                formula = a
-                    .decode_and_unescape_value(reader.decoder())
-                    .map_err(OdsError::Xml)?
-                    .to_string();
+                formula = a.decode_and_unescape_value(reader.decoder())?.to_string();
             }
             _ => (),
         }
@@ -658,38 +644,37 @@ where
         loop {
             buf.clear();
             match reader.read_event_into(buf) {
-                Ok(Event::Text(ref e)) => {
-                    s.push_str(&e.unescape()?);
+                Ok(Event::Text(t)) => {
+                    s.push_str(&t.xml10_content()?);
                 }
-                Ok(Event::End(ref e))
+                Ok(Event::GeneralRef(e)) => {
+                    unescape_entity_to_buffer(&e, &mut s)?;
+                }
+                Ok(Event::End(e))
                     if e.name() == QName(b"table:table-cell")
                         || e.name() == QName(b"table:covered-table-cell") =>
                 {
                     return Ok((Data::String(s), formula, true));
                 }
-                Ok(Event::Start(ref e)) if e.name() == QName(b"office:annotation") => loop {
+                Ok(Event::Start(e)) if e.name() == QName(b"office:annotation") => loop {
                     match reader.read_event_into(buf) {
-                        Ok(Event::End(ref e)) if e.name() == QName(b"office:annotation") => {
+                        Ok(Event::End(e)) if e.name() == QName(b"office:annotation") => {
                             break;
                         }
                         Err(e) => return Err(OdsError::Xml(e)),
                         _ => (),
                     }
                 },
-                Ok(Event::Start(ref e)) if e.name() == QName(b"text:p") => {
+                Ok(Event::Start(e)) if e.name() == QName(b"text:p") => {
                     if first_paragraph {
                         first_paragraph = false;
                     } else {
                         s.push('\n');
                     }
                 }
-                Ok(Event::Start(ref e)) if e.name() == QName(b"text:s") => {
+                Ok(Event::Start(e)) if e.name() == QName(b"text:s") => {
                     let count = match e.try_get_attribute("text:c")? {
-                        Some(c) => c
-                            .decode_and_unescape_value(reader.decoder())
-                            .map_err(OdsError::Xml)?
-                            .parse()
-                            .map_err(OdsError::ParseInt)?,
+                        Some(c) => c.decode_and_unescape_value(reader.decoder())?.parse()?,
                         None => 1,
                     };
                     for _ in 0..count {
@@ -717,41 +702,35 @@ where
     loop {
         buf.clear();
         match reader.read_event_into(&mut buf) {
-            Ok(Event::Start(ref e))
+            Ok(Event::Start(e))
                 if e.name() == QName(b"table:named-range")
                     || e.name() == QName(b"table:named-expression") =>
             {
                 let mut name = String::new();
                 let mut formula = String::new();
                 for a in e.attributes() {
-                    let a = a.map_err(OdsError::XmlAttr)?;
+                    let a = a?;
                     match a.key {
                         QName(b"table:name") => {
-                            name = a
-                                .decode_and_unescape_value(reader.decoder())
-                                .map_err(OdsError::Xml)?
-                                .to_string();
+                            name = a.decode_and_unescape_value(reader.decoder())?.to_string();
                         }
                         QName(b"table:cell-range-address" | b"table:expression") => {
-                            formula = a
-                                .decode_and_unescape_value(reader.decoder())
-                                .map_err(OdsError::Xml)?
-                                .to_string();
+                            formula = a.decode_and_unescape_value(reader.decoder())?.to_string();
                         }
                         _ => (),
                     }
                 }
                 defined_names.push((name, formula));
             }
-            Ok(Event::End(ref e))
+            Ok(Event::End(e))
                 if e.name() == QName(b"table:named-range")
                     || e.name() == QName(b"table:named-expression") => {}
-            Ok(Event::End(ref e)) if e.name() == QName(b"table:named-expressions") => break,
+            Ok(Event::End(e)) if e.name() == QName(b"table:named-expressions") => break,
             Err(e) => return Err(OdsError::Xml(e)),
             Ok(e) => {
                 return Err(OdsError::Mismatch {
                     expected: "table:named-expressions",
-                    found: format!("{:?}", e),
+                    found: format!("{e:?}"),
                 });
             }
         }
@@ -759,8 +738,9 @@ where
     Ok(defined_names)
 }
 
-/// Read pictures
+// Read pictures.
 #[cfg(feature = "picture")]
+#[allow(clippy::type_complexity)]
 fn read_pictures<RS: Read + Seek>(
     zip: &mut ZipArchive<RS>,
 ) -> Result<Option<Vec<(String, Vec<u8>)>>, OdsError> {
@@ -770,7 +750,7 @@ fn read_pictures<RS: Read + Seek>(
         let zname = zfile.name();
         // no Thumbnails
         if zname.starts_with("Pictures") {
-            if let Some(ext) = zname.split('.').last() {
+            if let Some(ext) = zname.split('.').next_back() {
                 if [
                     "emf", "wmf", "pict", "jpeg", "jpg", "png", "dib", "gif", "tiff", "eps", "bmp",
                     "wpg",
