@@ -1,9 +1,11 @@
-use std::borrow::Cow;
+// SPDX-License-Identifier: MIT
+//
+// Copyright 2016-2025, Johann Tuffe.
+
 use std::cmp::min;
 use std::collections::BTreeMap;
-use std::fmt::Write;
+use std::fmt::{self, Write};
 use std::io::{Read, Seek, SeekFrom};
-use std::marker::PhantomData;
 
 use log::debug;
 
@@ -67,9 +69,12 @@ pub enum XlsError {
     Etpg(u8),
     /// No vba project
     NoVba,
-    /// Invalid OfficeArt Record
+
+    /// Invalid OfficeArt Record.
     #[cfg(feature = "picture")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "picture")))]
     Art(&'static str),
+
     /// Worksheet not found
     WorksheetNotFound(String),
     /// Invalid iFmt value
@@ -98,7 +103,7 @@ impl std::fmt::Display for XlsError {
                 typ,
             } => write!(
                 f,
-                "Invalid {typ} length, expected {expected} maximum, found {found}",
+                "Invalid {typ} length, expected at least {expected}, found {found}",
             ),
             XlsError::ContinueRecordTooShort => write!(
                 f,
@@ -156,9 +161,9 @@ struct SheetData {
 /// A struct representing an old xls format file (CFB)
 pub struct Xls<RS> {
     sheets: BTreeMap<String, SheetData>,
-    vba: Option<VbaProject>,
     metadata: Metadata,
-    marker: PhantomData<RS>,
+    cfb: Cfb,
+    reader: RS,
     options: XlsOptions,
     formats: Vec<CellFormat>,
     is_1904: bool,
@@ -166,13 +171,20 @@ pub struct Xls<RS> {
     pictures: Option<Vec<(String, Vec<u8>)>>,
 }
 
+fn cfb<RS: Seek + Read>(reader: &mut RS) -> Result<Cfb, XlsError> {
+    let offset_end = reader.seek(SeekFrom::End(0))? as usize;
+    reader.seek(SeekFrom::Start(0))?;
+    let cfb = Cfb::new(reader, offset_end)?;
+    Ok(cfb)
+}
+
 impl<RS: Read + Seek> Xls<RS> {
     /// Creates a new instance using `Options` to inform parsing.
     ///
     /// ```
     /// use calamine::{Xls,XlsOptions};
     /// # use std::io::Cursor;
-    /// # const BYTES: &'static [u8] = b"";
+    /// # const BYTES: &[u8] = b"";
     ///
     /// # fn run() -> Result<Xls<Cursor<&'static [u8]>>, calamine::XlsError> {
     /// # let reader = std::io::Cursor::new(BYTES);
@@ -183,27 +195,14 @@ impl<RS: Read + Seek> Xls<RS> {
     /// # fn main() { assert!(run().is_err()); }
     /// ```
     pub fn new_with_options(mut reader: RS, options: XlsOptions) -> Result<Self, XlsError> {
-        let mut cfb = {
-            let offset_end = reader.seek(SeekFrom::End(0))? as usize;
-            reader.seek(SeekFrom::Start(0))?;
-            Cfb::new(&mut reader, offset_end)?
-        };
+        let cfb = cfb(&mut reader)?;
 
         debug!("cfb loaded");
 
-        // Reads vba once for all (better than reading all worksheets once for all)
-        let vba = if cfb.has_directory("_VBA_PROJECT_CUR") {
-            Some(VbaProject::from_cfb(&mut reader, &mut cfb)?)
-        } else {
-            None
-        };
-
-        debug!("vba ok");
-
         let mut xls = Xls {
             sheets: BTreeMap::new(),
-            vba,
-            marker: PhantomData,
+            cfb,
+            reader,
             metadata: Metadata::default(),
             options,
             is_1904: false,
@@ -212,7 +211,7 @@ impl<RS: Read + Seek> Xls<RS> {
             pictures: None,
         };
 
-        xls.parse_workbook(reader, cfb)?;
+        xls.parse_workbook()?;
 
         debug!("xls parsed");
 
@@ -225,7 +224,7 @@ impl<RS: Read + Seek> Xls<RS> {
     }
 
     /// Get the nth worksheet. Shortcut for getting the nth
-    /// sheet_name, then the corresponding worksheet.
+    /// sheet name, then the corresponding worksheet.
     pub fn worksheet_merge_cells_at(&self, n: usize) -> Option<Vec<Dimensions>> {
         let sheet = self.metadata().sheets.get(n)?;
 
@@ -245,8 +244,13 @@ impl<RS: Read + Seek> Reader<RS> for Xls<RS> {
         self
     }
 
-    fn vba_project(&mut self) -> Option<Result<Cow<'_, VbaProject>, XlsError>> {
-        self.vba.as_ref().map(|vba| Ok(Cow::Borrowed(vba)))
+    fn vba_project(&mut self) -> Result<Option<VbaProject>, XlsError> {
+        // Reads vba once for all (better than reading all worksheets once for all)
+        if !self.cfb.has_directory("_VBA_PROJECT_CUR") {
+            return Ok(None);
+        }
+        let vba = VbaProject::from_cfb(&mut self.reader, &mut self.cfb)?;
+        Ok(Some(vba))
     }
 
     /// Parses Workbook stream, no need for the relationships variable
@@ -302,11 +306,12 @@ struct Xti {
 }
 
 impl<RS: Read + Seek> Xls<RS> {
-    fn parse_workbook(&mut self, mut reader: RS, mut cfb: Cfb) -> Result<(), XlsError> {
+    fn parse_workbook(&mut self) -> Result<(), XlsError> {
         // gets workbook and worksheets stream, or early exit
-        let stream = cfb
-            .get_stream("Workbook", &mut reader)
-            .or_else(|_| cfb.get_stream("Book", &mut reader))?;
+        let stream = self
+            .cfb
+            .get_stream("Workbook", &mut self.reader)
+            .or_else(|_| self.cfb.get_stream("Book", &mut self.reader))?;
 
         let mut sheet_names = Vec::new();
         let mut strings = Vec::new();
@@ -330,7 +335,7 @@ impl<RS: Read + Seek> Xls<RS> {
                     // CodePage
                     0x0042 => {
                         if self.options.force_codepage.is_none() {
-                            encoding = XlsEncoding::from_codepage(read_u16(r.data))?
+                            encoding = XlsEncoding::from_codepage(read_u16(r.data))?;
                         }
                     }
                     0x013D => {
@@ -341,7 +346,7 @@ impl<RS: Read + Seek> Xls<RS> {
                     // Date1904
                     0x0022 => {
                         if read_u16(r.data) == 1 {
-                            self.is_1904 = true
+                            self.is_1904 = true;
                         }
                     }
                     // 2.4.126 FORMATTING
@@ -390,9 +395,7 @@ impl<RS: Read + Seek> Xls<RS> {
                     0x00EB => {
                         // MsoDrawingGroup
                         draw_group.extend(r.data);
-                        if let Some(cont) = r.cont {
-                            draw_group.extend(cont.iter().flat_map(|v| *v));
-                        }
+                        draw_group.extend(r.cont.iter().flat_map(|v| *v));
                     }
                     0x000A => break, // EOF,
                     _ => (),
@@ -424,7 +427,7 @@ impl<RS: Read + Seek> Xls<RS> {
             })
             .collect::<Vec<_>>();
 
-        debug!("defined_names: {:?}", defined_names);
+        debug!("defined_names: {defined_names:?}");
 
         let mut sheets = BTreeMap::new();
         let fmla_sheet_names = sheet_names
@@ -450,12 +453,12 @@ impl<RS: Read + Seek> Xls<RS> {
                     }
                     //0x0201 => cells.push(parse_blank(r.data)?), // 513: Blank
                     0x0203 => cells.push(parse_number(r.data, &self.formats, self.is_1904)?), // 515: Number
-                    0x0204 => cells.extend(parse_label(r.data, &encoding, biff)?), // 516: Label [MS-XLS 2.4.148]
-                    0x0205 => cells.push(parse_bool_err(r.data)?),                 // 517: BoolErr
+                    0x0204 => cells.push(parse_label(r.data, &encoding, biff)?), // 516: Label [MS-XLS 2.4.148]
+                    0x0205 => cells.push(parse_bool_err(r.data)?),               // 517: BoolErr
                     0x0207 => {
                         // 519 String (formula value)
                         let val = Data::String(parse_string(r.data, &encoding, biff)?);
-                        cells.push(Cell::new(fmla_pos, val))
+                        cells.push(Cell::new(fmla_pos, val));
                     }
                     0x027E => cells.push(parse_rk(r.data, &self.formats, self.is_1904)?), // 638: Rk
                     0x00FD => cells.extend(parse_label_sst(r.data, &strings)?), // LabelSst
@@ -487,11 +490,10 @@ impl<RS: Read + Seek> Xls<RS> {
                             &encoding,
                         )
                         .unwrap_or_else(|e| {
-                            debug!("{}", e);
+                            debug!("{e}");
                             format!(
                                 "Unrecognised formula \
-                                 for cell ({}, {}): {:?}",
-                                row, col, e
+                                 for cell ({row}, {col}): {e:?}"
                             )
                         });
                         formulas.push(Cell::new(fmla_pos, fmla));
@@ -526,14 +528,14 @@ impl<RS: Read + Seek> Xls<RS> {
     }
 }
 
-/// https://learn.microsoft.com/en-us/openspecs/office_file_formats/ms-xls/4d6a3d1e-d7c5-405f-bbae-d01e9cb79366
+/// <https://learn.microsoft.com/en-us/openspecs/office_file_formats/ms-xls/4d6a3d1e-d7c5-405f-bbae-d01e9cb79366>
 struct Bof {
     /// Binary Interchange File Format
     biff: Biff,
 }
 
-/// https://www.loc.gov/preservation/digital/formats/fdd/fdd000510.shtml#notes
-#[derive(Clone, Copy)]
+/// <https://www.loc.gov/preservation/digital/formats/fdd/fdd000510.shtml#notes>
+#[derive(Clone, Copy, Debug, PartialEq, Eq)]
 enum Biff {
     Biff2,
     Biff3,
@@ -551,7 +553,7 @@ fn parse_bof(r: &mut Record<'_>) -> Result<Bof, XlsError> {
 
     if r.data.len() >= 4 {
         dt = read_u16(&r.data[2..]);
-    };
+    }
 
     let biff = match biff_version {
         0x0200 | 0x0002 | 0x0007 => Biff::Biff2,
@@ -572,7 +574,7 @@ fn parse_bof(r: &mut Record<'_>) -> Result<Bof, XlsError> {
     Ok(Bof { biff })
 }
 
-/// BoundSheet8 [MS-XLS 2.4.28]
+/// `BoundSheet8` [MS-XLS 2.4.28]
 fn parse_sheet_metadata(
     r: &mut Record<'_>,
     encoding: &XlsEncoding,
@@ -605,7 +607,7 @@ fn parse_sheet_metadata(
     r.data = &r.data[6..];
     let mut name = parse_short_string(r, encoding, biff)?;
     name.retain(|c| c != '\0');
-    Ok((pos, Sheet { name, visible, typ }))
+    Ok((pos, Sheet { name, typ, visible }))
 }
 
 fn parse_number(r: &[u8], formats: &[CellFormat], is_1904: bool) -> Result<Cell<Data>, XlsError> {
@@ -693,7 +695,7 @@ fn parse_merge_cells(r: &[u8], merge_cells: &mut Vec<Dimensions>) -> Result<(),
         merge_cells.push(Dimensions {
             start: (rf.into(), cf.into()),
             end: (rl.into(), cl.into()),
-        })
+        });
     }
 
     Ok(())
@@ -755,7 +757,7 @@ fn rk_num(rk: &[u8], formats: &[CellFormat], is_1904: bool) -> Data {
     }
 }
 
-/// ShortXLUnicodeString [MS-XLS 2.5.240]
+/// `ShortXLUnicodeString` [MS-XLS 2.5.240]
 fn parse_short_string(
     r: &mut Record<'_>,
     encoding: &XlsEncoding,
@@ -779,36 +781,37 @@ fn parse_short_string(
     }
 
     let mut s = String::with_capacity(cch);
-    let _ = encoding.decode_to(r.data, cch, &mut s, high_byte);
+    encoding.decode_to(r.data, cch, &mut s, high_byte);
     Ok(s)
 }
 
-/// XLUnicodeString [MS-XLS 2.5.294]
+/// `XLUnicodeString` [MS-XLS 2.5.294]
 fn parse_string(r: &[u8], encoding: &XlsEncoding, biff: Biff) -> Result<String, XlsError> {
-    if r.len() < 4 {
+    let (mut high_byte, expected) = match biff {
+        Biff::Biff2 | Biff::Biff3 | Biff::Biff4 | Biff::Biff5 => (None, 2),
+        Biff::Biff8 => (Some(false), 3),
+    };
+    if r.len() < expected {
+        if 2 == r.len() && read_u16(r) == 0 {
+            // tests/OOM_alloc2.xls
+            return Ok(String::new());
+        }
         return Err(XlsError::Len {
             typ: "string",
-            expected: 4,
+            expected,
             found: r.len(),
         });
     }
+    // delay populating Some(_) variant until length checks guarantee r[2] can't crash
+    high_byte = high_byte.map(|_| r[2] & 0x1 != 0);
+
     let cch = read_u16(r) as usize;
-
-    let (high_byte, start) = match biff {
-        Biff::Biff2 | Biff::Biff3 | Biff::Biff4 | Biff::Biff5 => (None, 2),
-        _ => (Some(r[2] & 0x1 != 0), 3),
-    };
-
     let mut s = String::with_capacity(cch);
-    let _ = encoding.decode_to(&r[start..], cch, &mut s, high_byte);
+    encoding.decode_to(&r[expected..], cch, &mut s, high_byte);
     Ok(s)
 }
 
-fn parse_label(
-    r: &[u8],
-    encoding: &XlsEncoding,
-    biff: Biff,
-) -> Result<Option<Cell<Data>>, XlsError> {
+fn parse_label(r: &[u8], encoding: &XlsEncoding, biff: Biff) -> Result<Cell<Data>, XlsError> {
     if r.len() < 6 {
         return Err(XlsError::Len {
             typ: "label",
@@ -819,10 +822,10 @@ fn parse_label(
     let row = read_u16(r);
     let col = read_u16(&r[2..]);
     let _ixfe = read_u16(&r[4..]);
-    Ok(Some(Cell::new(
+    Ok(Cell::new(
         (row as u32, col as u32),
         Data::String(parse_string(&r[6..], encoding, biff)?),
-    )))
+    ))
 }
 
 fn parse_label_sst(r: &[u8], strings: &[String]) -> Result<Option<Cell<Data>>, XlsError> {
@@ -848,7 +851,7 @@ fn parse_label_sst(r: &[u8], strings: &[String]) -> Result<Option<Cell<Data>>, X
 }
 
 fn parse_dimensions(r: &[u8]) -> Result<Dimensions, XlsError> {
-    let (rf, rl, cf, cl) = match r.len() {
+    let (rf, rl, mut cf, cl) = match r.len() {
         10 => (
             read_u16(&r[0..2]) as u32,
             read_u16(&r[2..4]) as u32,
@@ -869,6 +872,12 @@ fn parse_dimensions(r: &[u8]) -> Result<Dimensions, XlsError> {
             });
         }
     };
+    // 2.5.53 ColU must be <= 0xFF, if larger, reasonable to assume
+    // starts at 0
+    // tests/OOM_alloc2.xls
+    if 0xFF < cf || cl < cf {
+        cf = 0;
+    }
     if 1 <= rl && 1 <= cl {
         Ok(Dimensions {
             start: (rf, cf),
@@ -882,6 +891,9 @@ fn parse_dimensions(r: &[u8]) -> Result<Dimensions, XlsError> {
     }
 }
 
+// Parse the Excel xls Shared String Table (SST). See [MS-XLS] 2.4.265.
+//
+// https://learn.microsoft.com/en-us/openspecs/office_file_formats/ms-xls/b6231b92-d32e-4626-badd-c3310a672bab
 fn parse_sst(r: &mut Record<'_>, encoding: &XlsEncoding) -> Result<Vec<String>, XlsError> {
     if r.data.len() < 8 {
         return Err(XlsError::Len {
@@ -890,19 +902,21 @@ fn parse_sst(r: &mut Record<'_>, encoding: &XlsEncoding) -> Result<Vec<String>,
             found: r.data.len(),
         });
     }
-    let len: usize = read_i32(&r.data[4..8]).try_into().unwrap();
-    let mut sst = Vec::with_capacity(len);
+    let mut sst = vec![];
+
+    // Skip cstTotal and cstUnique headers in SST record.
     r.data = &r.data[8..];
 
-    for _ in 0..len {
+    while !r.data.is_empty() || r.continue_record() {
         sst.push(read_rich_extended_string(r, encoding)?);
     }
+
     Ok(sst)
 }
 
 /// Decode XF (extract only ifmt - Format identifier)
 ///
-/// See: https://learn.microsoft.com/ru-ru/openspecs/office_file_formats/ms-xls/993d15c4-ec04-43e9-ba36-594dfb336c6d
+/// See: <https://learn.microsoft.com/ru-ru/openspecs/office_file_formats/ms-xls/993d15c4-ec04-43e9-ba36-594dfb336c6d>
 fn parse_xf(r: &Record<'_>) -> Result<u16, XlsError> {
     if r.data.len() < 4 {
         return Err(XlsError::Len {
@@ -915,10 +929,9 @@ fn parse_xf(r: &Record<'_>) -> Result<u16, XlsError> {
     Ok(read_u16(&r.data[2..]))
 }
 
-/// Decode Format
+/// Decode Format [MS-XLS 2.4.126]
 ///
-/// See: https://learn.microsoft.com/ru-ru/openspecs/office_file_formats/ms-xls/300280fd-e4fe-4675-a924-4d383af48d3b
-/// 2.4.126
+/// See: <https://learn.microsoft.com/ru-ru/openspecs/office_file_formats/ms-xls/300280fd-e4fe-4675-a924-4d383af48d3b>
 fn parse_format(
     r: &mut Record<'_>,
     encoding: &XlsEncoding,
@@ -941,14 +954,18 @@ fn parse_format(
     Ok((ifmt, detect_custom_number_format(&s)))
 }
 
-/// Decode XLUnicodeRichExtendedString.
+/// Decode `XLUnicodeRichExtendedString` [MS-XLS 2.5.293].
 ///
 /// See: <https://docs.microsoft.com/en-us/openspecs/office_file_formats/ms-xls/173d9f51-e5d3-43da-8de2-be7f22e119b9>
 fn read_rich_extended_string(
     r: &mut Record<'_>,
     encoding: &XlsEncoding,
 ) -> Result<String, XlsError> {
-    if r.data.is_empty() && !r.continue_record() || r.data.len() < 3 {
+    if r.data.is_empty() {
+        // spec violation: at very least cch and flags should be present
+        return Ok(String::new());
+    }
+    if r.data.len() < 3 {
         return Err(XlsError::Len {
             typ: "rich extended string",
             expected: 3,
@@ -1023,21 +1040,16 @@ fn read_unicode_string_no_cch(encoding: &XlsEncoding, buf: &[u8], len: &usize, s
 struct Record<'a> {
     typ: u16,
     data: &'a [u8],
-    cont: Option<Vec<&'a [u8]>>,
+    cont: Vec<&'a [u8]>,
 }
 
 impl<'a> Record<'a> {
     fn continue_record(&mut self) -> bool {
-        match self.cont {
-            None => false,
-            Some(ref mut v) => {
-                if v.is_empty() {
-                    false
-                } else {
-                    self.data = v.remove(0);
-                    true
-                }
-            }
+        if self.cont.is_empty() {
+            false
+        } else {
+            self.data = self.cont.remove(0);
+            true
         }
     }
 
@@ -1055,6 +1067,29 @@ impl<'a> Record<'a> {
     }
 }
 
+// Simple Debug impl to dump record data in hex format.
+impl fmt::Debug for Record<'_> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        writeln!(
+            f,
+            "\nRecord = 0x{:04X}, Length = 0x{:04X}, {}",
+            self.typ,
+            self.data.len(),
+            self.data.len()
+        )?;
+
+        let mut iter = self.data.chunks(16);
+        for chunk in iter.by_ref() {
+            for byte in chunk {
+                write!(f, "{byte:02X} ")?;
+            }
+            writeln!(f)?;
+        }
+
+        Ok(())
+    }
+}
+
 struct RecordIter<'a> {
     stream: &'a [u8],
 }
@@ -1079,8 +1114,8 @@ impl<'a> Iterator for RecordIter<'a> {
         let d = &data[4..];
 
         // Append next record data if it is a Continue record
-        let cont = if next.len() > 4 && read_u16(next) == 0x003C {
-            let mut cont = Vec::new();
+        let mut cont = Vec::new();
+        if next.len() > 4 && read_u16(next) == 0x003C {
             while self.stream.len() > 4 && read_u16(self.stream) == 0x003C {
                 len = read_u16(&self.stream[2..]) as usize;
                 if self.stream.len() < len + 4 {
@@ -1090,10 +1125,7 @@ impl<'a> Iterator for RecordIter<'a> {
                 cont.push(&sp.0[4..]);
                 self.stream = sp.1;
             }
-            Some(cont)
-        } else {
-            None
-        };
+        }
 
         Some(Ok(Record {
             typ: t,
@@ -1132,12 +1164,12 @@ fn parse_defined_names(rgce: &[u8]) -> Result<(Option<usize>, String), XlsError>
             f.push('$');
             push_column(read_u16(&rgce[7..9]) as u32, &mut f);
             f.push('$');
-            f.push_str(&format!("{}", read_u16(&rgce[3..5]) as u32 + 1));
+            write!(&mut f, "{}", read_u16(&rgce[3..5]) as u32 + 1).unwrap();
             f.push(':');
             f.push('$');
             push_column(read_u16(&rgce[9..11]) as u32, &mut f);
             f.push('$');
-            f.push_str(&format!("{}", read_u16(&rgce[5..7]) as u32 + 1));
+            write!(&mut f, "{}", read_u16(&rgce[5..7]) as u32 + 1).unwrap();
             (Some(ixti), f)
         }
         0x3c | 0x5c | 0x7c | 0x3d | 0x5d | 0x7d => {
@@ -1145,14 +1177,14 @@ fn parse_defined_names(rgce: &[u8]) -> Result<(Option<usize>, String), XlsError>
             let ixti = read_u16(&rgce[1..3]) as usize;
             (Some(ixti), "#REF!".to_string())
         }
-        _ => (None, format!("Unsupported ptg: {:x}", ptg)),
+        _ => (None, format!("Unsupported ptg: {ptg:x}")),
     };
     Ok(res)
 }
 
 /// Formula parsing
 ///
-/// CellParsedFormula [MS-XLS 2.5.198.3]
+/// `CellParsedFormula` [MS-XLS 2.5.198.3]
 fn parse_formula(
     mut rgce: &[u8],
     sheets: &[String],
@@ -1252,7 +1284,7 @@ fn parse_formula(
                     _ => unreachable!(),
                 };
                 let e2 = formula.split_off(e2);
-                write!(&mut formula, "{}{}", op, e2).unwrap();
+                write!(&mut formula, "{op}{e2}").unwrap();
             }
             0x12 => {
                 let e = stack.last().ok_or(XlsError::StackLen)?;
@@ -1298,7 +1330,7 @@ fn parse_formula(
                         rgce = &rgce[2..];
                         let e = *stack.last().ok_or(XlsError::StackLen)?;
                         let e = formula.split_off(e);
-                        write!(&mut formula, "SUM({})", e).unwrap();
+                        write!(&mut formula, "SUM({e})").unwrap();
                     }
                     0x40 | 0x41 => {
                         // PtfAttrSpace
@@ -1430,7 +1462,7 @@ fn parse_formula(
                 if rgce[3] & 0x40 != 0x40 {
                     formula.push('$');
                 }
-                formula.push_str(&format!("{}", row));
+                formula.push_str(&format!("{row}"));
                 rgce = &rgce[4..];
             }
             0x25 | 0x45 | 0x65 => {
@@ -1475,7 +1507,7 @@ fn parse_formula(
     }
 }
 
-/// FormulaValue [MS-XLS 2.5.133]
+/// `FormulaValue` [MS-XLS 2.5.133]
 fn parse_formula_value(r: &[u8]) -> Result<Option<Data>, XlsError> {
     match *r {
         // String, value should be in next record
@@ -1492,7 +1524,7 @@ fn parse_formula_value(r: &[u8]) -> Result<Option<Data>, XlsError> {
     }
 }
 
-/// OfficeArtRecord [MS-ODRAW 1.3.1]
+// OfficeArtRecord [MS-ODRAW 1.3.1].
 #[cfg(feature = "picture")]
 struct ArtRecord<'a> {
     instance: u16,
@@ -1538,7 +1570,7 @@ impl<'a> Iterator for ArtRecordIter<'a> {
     }
 }
 
-/// Parsing pictures
+// Parsing pictures.
 #[cfg(feature = "picture")]
 fn parse_pictures(stream: &[u8]) -> Result<Vec<(String, Vec<u8>)>, XlsError> {
     let mut pics = Vec::new();
@@ -1630,3 +1662,14 @@ fn parse_pictures(stream: &[u8]) -> Result<Vec<(String, Vec<u8>)>, XlsError> {
     }
     Ok(pics)
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn test_parse_string() {
+        let enc = XlsEncoding::from_codepage(1252).unwrap();
+        parse_string(&[0, 1], &enc, Biff::Biff8).unwrap_err();
+    }
+}
