@@ -1,3 +1,7 @@
+// SPDX-License-Identifier: MIT
+//
+// Copyright 2016-2025, Johann Tuffe.
+
 mod cells_reader;
 
 pub use cells_reader::XlsbCellsReader;
@@ -56,7 +60,7 @@ pub enum XlsbError {
     Etpg(u8),
     /// Unsupported iftab
     IfTab(usize),
-    /// Unsupported BErr
+    /// Unsupported `BErr`
     BErr(u8),
     /// Unsupported Ptg
     Ptg(u8),
@@ -87,7 +91,9 @@ pub enum XlsbError {
 from_err!(std::io::Error, XlsbError, Io);
 from_err!(zip::result::ZipError, XlsbError, Zip);
 from_err!(quick_xml::Error, XlsbError, Xml);
+from_err!(quick_xml::events::attributes::AttrError, XlsbError, XmlAttr);
 from_err!(quick_xml::encoding::EncodingError, XlsbError, Encoding);
+from_err!(crate::vba::VbaError, XlsbError, Vba);
 
 impl std::fmt::Display for XlsbError {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
@@ -172,11 +178,11 @@ impl<RS: Read + Seek> Xlsb<RS> {
 
                 loop {
                     match xml.read_event_into(&mut buf) {
-                        Ok(Event::Start(ref e)) if e.name() == QName(b"Relationship") => {
+                        Ok(Event::Start(e)) if e.name() == QName(b"Relationship") => {
                             let mut id = None;
                             let mut target = None;
                             for a in e.attributes() {
-                                match a.map_err(XlsbError::XmlAttr)? {
+                                match a? {
                                     Attribute {
                                         key: QName(b"Id"),
                                         value: v,
@@ -348,7 +354,7 @@ impl<RS: Read + Seek> Xlsb<RS> {
                             visible,
                         });
                         self.sheets.push((name.into_owned(), path));
-                    };
+                    }
                 }
                 0x0090 => break, // BrtEndBundleShs
                 _ => (),
@@ -399,7 +405,7 @@ impl<RS: Read + Seek> Xlsb<RS> {
                     self.metadata.names = defined_names;
                     return Ok(());
                 }
-                _ => debug!("Unsupported type {:X}", typ),
+                _ => debug!("Unsupported type {typ:X}"),
             }
         }
     }
@@ -431,7 +437,7 @@ impl<RS: Read + Seek> Xlsb<RS> {
             let mut zfile = self.zip.by_index(i)?;
             let zname = zfile.name();
             if zname.starts_with("xl/media") {
-                if let Some(ext) = zname.split('.').last() {
+                if let Some(ext) = zname.split('.').next_back() {
                     if [
                         "emf", "wmf", "pict", "jpeg", "jpg", "png", "dib", "gif", "tiff", "eps",
                         "bmp", "wpg",
@@ -486,13 +492,13 @@ impl<RS: Read + Seek> Reader<RS> for Xlsb<RS> {
         self
     }
 
-    fn vba_project(&mut self) -> Option<Result<Cow<'_, VbaProject>, XlsbError>> {
-        self.zip.by_name("xl/vbaProject.bin").ok().map(|mut f| {
-            let len = f.size() as usize;
-            VbaProject::new(&mut f, len)
-                .map(Cow::Owned)
-                .map_err(XlsbError::Vba)
-        })
+    fn vba_project(&mut self) -> Result<Option<VbaProject>, XlsbError> {
+        let Some(mut f) = self.zip.by_name("xl/vbaProject.bin").ok() else {
+            return Ok(None);
+        };
+        let len = f.size() as usize;
+        let vba = VbaProject::new(&mut f, len)?;
+        Ok(Some(vba))
     }
 
     fn metadata(&self) -> &Metadata {
@@ -589,7 +595,7 @@ impl<RS: Read + Seek> ReaderRef<RS> for Xlsb<RS> {
 
                 // If `header_row` is set and the first non-empty cell is not at the `header_row`, we add
                 // an empty cell at the beginning with row `header_row` and same column as the first non-empty cell.
-                if cells.first().map_or(false, |c| c.pos.0 != header_row_idx) {
+                if cells.first().is_some_and(|c| c.pos.0 != header_row_idx) {
                     cells.insert(
                         0,
                         Cell {
@@ -621,7 +627,9 @@ where
     RS: Read + Seek,
 {
     fn from_zip(zip: &'a mut ZipArchive<RS>, path: &str) -> Result<RecordIter<'a, RS>, XlsbError> {
-        match zip.by_name(path) {
+        let zip_path = crate::xlsx::path_to_zip_path(zip, path);
+
+        match zip.by_name(&zip_path) {
             Ok(f) => Ok(RecordIter {
                 r: BufReader::new(f),
                 b: [0],
@@ -958,7 +966,7 @@ fn parse_formula(
                 if rgce[5] & 0x40 != 0x40 {
                     formula.push('$');
                 }
-                formula.push_str(&format!("{}", row));
+                formula.push_str(&format!("{row}"));
                 rgce = &rgce[6..];
             }
             0x25 | 0x45 | 0x65 => {
@@ -1026,7 +1034,7 @@ fn check_for_password_protected<RS: Read + Seek>(reader: &mut RS) -> Result<(),
         if cfb.has_directory("EncryptedPackage") {
             return Err(XlsbError::Password);
         }
-    };
+    }
 
     Ok(())
 }
