@@ -1,3 +1,7 @@
+// SPDX-License-Identifier: MIT
+//
+// Copyright 2016-2025, Johann Tuffe.
+
 use quick_xml::{
     events::{attributes::Attribute, BytesStart, Event},
     name::QName,
@@ -15,6 +19,7 @@ use super::{
 use crate::{
     datatype::DataRef,
     formats::{format_excel_f64_ref, CellFormat},
+    utils::unescape_entity_to_buffer,
     Cell, XlsxError,
 };
 
@@ -53,13 +58,13 @@ where
         'xml: loop {
             buf.clear();
             match xml.read_event_into(&mut buf).map_err(XlsxError::Xml)? {
-                Event::Start(ref e) => match e.local_name().as_ref() {
+                Event::Start(e) => match e.local_name().as_ref() {
                     b"dimension" => {
                         for a in e.attributes() {
                             if let Attribute {
                                 key: QName(b"ref"),
                                 value: rdim,
-                            } = a.map_err(XlsxError::XmlAttr)?
+                            } = a?
                             {
                                 dimensions = get_dimension(&rdim)?;
                                 continue 'xml;
@@ -106,20 +111,18 @@ where
         loop {
             self.buf.clear();
             match self.xml.read_event_into(&mut self.buf) {
-                Ok(Event::Start(ref row_element))
-                    if row_element.local_name().as_ref() == b"row" =>
-                {
+                Ok(Event::Start(row_element)) if row_element.local_name().as_ref() == b"row" => {
                     let attribute = get_attribute(row_element.attributes(), QName(b"r"))?;
                     if let Some(range) = attribute {
                         let row = get_row(range)?;
                         self.row_index = row;
                     }
                 }
-                Ok(Event::End(ref row_element)) if row_element.local_name().as_ref() == b"row" => {
+                Ok(Event::End(row_element)) if row_element.local_name().as_ref() == b"row" => {
                     self.row_index += 1;
                     self.col_index = 0;
                 }
-                Ok(Event::Start(ref c_element)) if c_element.local_name().as_ref() == b"c" => {
+                Ok(Event::Start(c_element)) if c_element.local_name().as_ref() == b"c" => {
                     let attribute = get_attribute(c_element.attributes(), QName(b"r"))?;
                     let pos = if let Some(range) = attribute {
                         let (row, col) = get_row_column(range)?;
@@ -132,17 +135,17 @@ where
                     loop {
                         self.cell_buf.clear();
                         match self.xml.read_event_into(&mut self.cell_buf) {
-                            Ok(Event::Start(ref e)) => {
+                            Ok(Event::Start(e)) => {
                                 value = read_value(
                                     self.strings,
                                     self.formats,
                                     self.is_1904,
                                     &mut self.xml,
-                                    e,
-                                    c_element,
-                                )?
+                                    &e,
+                                    &c_element,
+                                )?;
                             }
-                            Ok(Event::End(ref e)) if e.local_name().as_ref() == b"c" => break,
+                            Ok(Event::End(e)) if e.local_name().as_ref() == b"c" => break,
                             Ok(Event::Eof) => return Err(XlsxError::XmlEof("c")),
                             Err(e) => return Err(XlsxError::Xml(e)),
                             _ => (),
@@ -151,7 +154,7 @@ where
                     self.col_index += 1;
                     return Ok(Some(Cell::new(pos, value)));
                 }
-                Ok(Event::End(ref e)) if e.local_name().as_ref() == b"sheetData" => {
+                Ok(Event::End(e)) if e.local_name().as_ref() == b"sheetData" => {
                     return Ok(None);
                 }
                 Ok(Event::Eof) => return Err(XlsxError::XmlEof("sheetData")),
@@ -165,20 +168,18 @@ where
         loop {
             self.buf.clear();
             match self.xml.read_event_into(&mut self.buf) {
-                Ok(Event::Start(ref row_element))
-                    if row_element.local_name().as_ref() == b"row" =>
-                {
+                Ok(Event::Start(row_element)) if row_element.local_name().as_ref() == b"row" => {
                     let attribute = get_attribute(row_element.attributes(), QName(b"r"))?;
                     if let Some(range) = attribute {
                         let row = get_row(range)?;
                         self.row_index = row;
                     }
                 }
-                Ok(Event::End(ref row_element)) if row_element.local_name().as_ref() == b"row" => {
+                Ok(Event::End(row_element)) if row_element.local_name().as_ref() == b"row" => {
                     self.row_index += 1;
                     self.col_index = 0;
                 }
-                Ok(Event::Start(ref c_element)) if c_element.local_name().as_ref() == b"c" => {
+                Ok(Event::Start(c_element)) if c_element.local_name().as_ref() == b"c" => {
                     let attribute = get_attribute(c_element.attributes(), QName(b"r"))?;
                     let pos = if let Some(range) = attribute {
                         let (row, col) = get_row_column(range)?;
@@ -191,8 +192,8 @@ where
                     loop {
                         self.cell_buf.clear();
                         match self.xml.read_event_into(&mut self.cell_buf) {
-                            Ok(Event::Start(ref e)) => {
-                                let formula = read_formula(&mut self.xml, e)?;
+                            Ok(Event::Start(e)) => {
+                                let formula = read_formula(&mut self.xml, &e)?;
                                 if let Some(f) = formula.borrow() {
                                     value = Some(f.clone());
                                 }
@@ -222,39 +223,27 @@ where
                                     // shared reference
                                     match get_attribute(e.attributes(), QName(b"ref"))? {
                                         Some(res) => {
-                                            // orignal reference formula
+                                            // original reference formula
                                             let reference = get_dimension(res)?;
-                                            if reference.start.0 != reference.end.0 {
-                                                for i in 0..=(reference.end.0 - reference.start.0) {
+
+                                            for row in reference.start.0..=reference.end.0 {
+                                                for column in reference.start.1..=reference.end.1 {
                                                     offset_map.insert(
-                                                        (reference.start.0 + i, reference.start.1),
+                                                        (row, column),
                                                         (
-                                                            (reference.start.0 as i64
-                                                                - pos.0 as i64
-                                                                + i as i64),
-                                                            0,
-                                                        ),
-                                                    );
-                                                }
-                                            } else if reference.start.1 != reference.end.1 {
-                                                for i in 0..=(reference.end.1 - reference.start.1) {
-                                                    offset_map.insert(
-                                                        (reference.start.0, reference.start.1 + i),
-                                                        (
-                                                            0,
-                                                            (reference.start.1 as i64
-                                                                - pos.1 as i64
-                                                                + i as i64),
+                                                            row as i64 - pos.0 as i64,
+                                                            column as i64 - pos.1 as i64,
                                                         ),
                                                     );
                                                 }
                                             }
 
                                             if let Some(f) = formula.borrow() {
-                                                while self.formulas.len() < shared_index {
-                                                    self.formulas.push(None);
+                                                if self.formulas.len() <= shared_index {
+                                                    self.formulas.resize(shared_index + 1, None);
                                                 }
-                                                self.formulas.push(Some((f.clone(), offset_map)));
+                                                self.formulas[shared_index] =
+                                                    Some((f.clone(), offset_map));
                                             }
                                             value = formula;
                                         }
@@ -268,10 +257,10 @@ where
                                                 }
                                             }
                                         }
-                                    };
-                                };
+                                    }
+                                }
                             }
-                            Ok(Event::End(ref e)) if e.local_name().as_ref() == b"c" => break,
+                            Ok(Event::End(e)) if e.local_name().as_ref() == b"c" => break,
                             Ok(Event::Eof) => return Err(XlsxError::XmlEof("c")),
                             Err(e) => return Err(XlsxError::Xml(e)),
                             _ => (),
@@ -280,7 +269,7 @@ where
                     self.col_index += 1;
                     return Ok(Some(Cell::new(pos, value.unwrap_or_default())));
                 }
-                Ok(Event::End(ref e)) if e.local_name().as_ref() == b"sheetData" => {
+                Ok(Event::End(e)) if e.local_name().as_ref() == b"sheetData" => {
                     return Ok(None);
                 }
                 Ok(Event::Eof) => return Err(XlsxError::XmlEof("sheetData")),
@@ -314,7 +303,8 @@ where
             loop {
                 v_buf.clear();
                 match xml.read_event_into(&mut v_buf)? {
-                    Event::Text(t) => v.push_str(&t.unescape()?),
+                    Event::Text(t) => v.push_str(&t.xml10_content()?),
+                    Event::GeneralRef(e) => unescape_entity_to_buffer(&e, &mut v)?,
                     Event::End(end) if end.name() == e.name() => break,
                     Event::Eof => return Err(XlsxError::XmlEof("v")),
                     _ => (),
@@ -347,9 +337,14 @@ fn read_v<'s>(
     };
     match get_attribute(c_element.attributes(), QName(b"t"))? {
         Some(b"s") => {
-            // shared string
+            // Cell value is an index into the shared string table.
             let idx = atoi_simd::parse::<usize>(v.as_bytes()).unwrap_or(0);
-            Ok(DataRef::SharedString(&strings[idx]))
+            match strings.get(idx) {
+                Some(shared_string) => Ok(DataRef::SharedString(shared_string)),
+                None => Err(XlsxError::Unexpected(
+                    "Cell string index not found in shared strings table",
+                )),
+            }
         }
         Some(b"b") => {
             // boolean
@@ -412,7 +407,8 @@ where
             let mut f = String::new();
             loop {
                 match xml.read_event_into(&mut f_buf)? {
-                    Event::Text(t) => f.push_str(&t.unescape()?),
+                    Event::Text(t) => f.push_str(&t.xml10_content()?),
+                    Event::GeneralRef(e) => unescape_entity_to_buffer(&e, &mut f)?,
                     Event::End(end) if end.name() == e.name() => break,
                     Event::Eof => return Err(XlsxError::XmlEof("f")),
                     _ => (),
