@@ -4,7 +4,6 @@ mod box_;
 mod byte_slice;
 mod offset;
 mod option;
-mod planus_vectors;
 mod primitives;
 mod ref_;
 mod result;
@@ -12,10 +11,11 @@ mod slice;
 mod str;
 mod string;
 mod union_offset;
+mod union_vector_offset;
 mod unit;
 mod vec;
 
-fn array_from_buffer(
+pub(crate) fn array_from_buffer(
     buffer: crate::slice_helpers::SliceWithStartOffset<'_>,
     offset: usize,
 ) -> core::result::Result<
