@@ -1,7 +1,7 @@
 #![allow(unsafe_op_in_unsafe_fn)]
 use std::ops::Deref;
 
-use bytemuck::Zeroable;
+use bytemuck::{Pod, Zeroable};
 use either::Either;
 
 use super::IntoIter;
@@ -37,7 +37,6 @@ use crate::storage::SharedStorage;
 /// // but cloning forbids getting mut since `slice` and `buffer` now share data
 /// assert_eq!(buffer.get_mut_slice(), None);
 /// ```
-#[derive(Clone)]
 pub struct Buffer<T> {
     /// The internal byte buffer.
     storage: SharedStorage<T>,
@@ -49,6 +48,16 @@ pub struct Buffer<T> {
     length: usize,
 }
 
+impl<T> Clone for Buffer<T> {
+    fn clone(&self) -> Self {
+        Self {
+            storage: self.storage.clone(),
+            ptr: self.ptr,
+            length: self.length,
+        }
+    }
+}
+
 unsafe impl<T: Send + Sync> Sync for Buffer<T> {}
 unsafe impl<T: Send + Sync> Send for Buffer<T> {}
 
@@ -270,6 +279,25 @@ impl<T> Buffer<T> {
     }
 }
 
+impl<T: Pod> Buffer<T> {
+    pub fn try_transmute<U: Pod>(mut self) -> Result<Buffer<U>, Self> {
+        assert_ne!(size_of::<U>(), 0);
+        let ptr = self.ptr as *const U;
+        let length = self.length;
+        match self.storage.try_transmute() {
+            Err(v) => {
+                self.storage = v;
+                Err(self)
+            },
+            Ok(storage) => Ok(Buffer {
+                storage,
+                ptr,
+                length: length.checked_mul(size_of::<T>()).expect("overflow") / size_of::<U>(),
+            }),
+        }
+    }
+}
+
 impl<T: Clone> Buffer<T> {
     pub fn make_mut(self) -> Vec<T> {
         match self.into_mut() {
@@ -360,3 +388,49 @@ impl<T> Splitable for Buffer<T> {
         )
     }
 }
+
+#[cfg(feature = "serde")]
+mod _serde_impl {
+    use serde::{Deserialize, Serialize};
+
+    use super::Buffer;
+
+    impl<T> Serialize for Buffer<T>
+    where
+        T: Serialize,
+    {
+        fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+        where
+            S: serde::Serializer,
+        {
+            <[T] as Serialize>::serialize(self.as_slice(), serializer)
+        }
+    }
+
+    impl<'de, T> Deserialize<'de> for Buffer<T>
+    where
+        T: Deserialize<'de>,
+    {
+        fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+        where
+            D: serde::Deserializer<'de>,
+        {
+            <Vec<T> as Deserialize>::deserialize(deserializer).map(Buffer::from)
+        }
+    }
+}
+
+#[cfg(feature = "dsl-schema")]
+impl<T: schemars::JsonSchema> schemars::JsonSchema for Buffer<T> {
+    fn schema_name() -> String {
+        <[T] as schemars::JsonSchema>::schema_name()
+    }
+
+    fn schema_id() -> std::borrow::Cow<'static, str> {
+        <[T] as schemars::JsonSchema>::schema_id()
+    }
+
+    fn json_schema(generator: &mut schemars::r#gen::SchemaGenerator) -> schemars::schema::Schema {
+        <[T] as schemars::JsonSchema>::json_schema(generator)
+    }
+}
