@@ -9,6 +9,8 @@ use arrow::offset::OffsetsBuffer;
 use arrow::temporal_conversions::*;
 use arrow::types::months_days_ns;
 use polars_compute::cast::cast_unchecked as cast;
+#[cfg(feature = "dtype-decimal")]
+use polars_compute::decimal::dec128_fits;
 use polars_error::feature_gated;
 use polars_utils::check_allow_importing_interval_as_struct;
 use polars_utils::itertools::Itertools;
@@ -48,9 +50,9 @@ impl Series {
         Ok(series)
     }
 
-    /// Takes chunks and a polars datatype and constructs the Series
+    /// Takes chunks and a polars datatype and constructs the Series.
     /// This is faster than creating from chunks and an arrow datatype because there is no
-    /// casting involved
+    /// casting involved.
     ///
     /// # Safety
     ///
@@ -72,6 +74,8 @@ impl Series {
             UInt64 => UInt64Chunked::from_chunks(name, chunks).into_series(),
             #[cfg(feature = "dtype-i128")]
             Int128 => Int128Chunked::from_chunks(name, chunks).into_series(),
+            #[cfg(feature = "dtype-u128")]
+            UInt128 => UInt128Chunked::from_chunks(name, chunks).into_series(),
             #[cfg(feature = "dtype-date")]
             Date => Int32Chunked::from_chunks(name, chunks)
                 .into_date()
@@ -90,10 +94,7 @@ impl Series {
                 .into_series(),
             #[cfg(feature = "dtype-decimal")]
             Decimal(precision, scale) => Int128Chunked::from_chunks(name, chunks)
-                .into_decimal_unchecked(
-                    *precision,
-                    scale.unwrap_or_else(|| unreachable!("scale should be set")),
-                )
+                .into_decimal_unchecked(*precision, *scale)
                 .into_series(),
             #[cfg(feature = "dtype-array")]
             Array(_, _) => {
@@ -218,6 +219,10 @@ impl Series {
             ArrowDataType::UInt16 => Ok(UInt16Chunked::from_chunks(name, chunks).into_series()),
             ArrowDataType::UInt32 => Ok(UInt32Chunked::from_chunks(name, chunks).into_series()),
             ArrowDataType::UInt64 => Ok(UInt64Chunked::from_chunks(name, chunks).into_series()),
+            ArrowDataType::UInt128 => feature_gated!(
+                "dtype-u128",
+                Ok(UInt128Chunked::from_chunks(name, chunks).into_series())
+            ),
             #[cfg(feature = "dtype-i8")]
             ArrowDataType::Int8 => Ok(Int8Chunked::from_chunks(name, chunks).into_series()),
             #[cfg(feature = "dtype-i16")]
@@ -300,8 +305,7 @@ impl Series {
             },
             ArrowDataType::Decimal32(precision, scale) => {
                 feature_gated!("dtype-decimal", {
-                    polars_ensure!(*scale <= *precision, InvalidOperation: "invalid decimal precision and scale (prec={precision}, scale={scale})");
-                    polars_ensure!(*precision <= 38, InvalidOperation: "polars does not support decimals above 38 precision");
+                    polars_compute::decimal::dec128_verify_prec_scale(*precision, *scale)?;
 
                     let mut chunks = chunks;
                     for chunk in chunks.iter_mut() {
@@ -320,17 +324,15 @@ impl Series {
                         .to_boxed();
                     }
 
-                    // @NOTE: We cannot cast here as that will lower the scale.
                     let s = Int128Chunked::from_chunks(name, chunks)
-                        .into_decimal_unchecked(Some(*precision), *scale)
+                        .into_decimal_unchecked(*precision, *scale)
                         .into_series();
                     Ok(s)
                 })
             },
             ArrowDataType::Decimal64(precision, scale) => {
                 feature_gated!("dtype-decimal", {
-                    polars_ensure!(*scale <= *precision, InvalidOperation: "invalid decimal precision and scale (prec={precision}, scale={scale})");
-                    polars_ensure!(*precision <= 38, InvalidOperation: "polars does not support decimals above 38 precision");
+                    polars_compute::decimal::dec128_verify_prec_scale(*precision, *scale)?;
 
                     let mut chunks = chunks;
                     for chunk in chunks.iter_mut() {
@@ -349,20 +351,16 @@ impl Series {
                         .to_boxed();
                     }
 
-                    // @NOTE: We cannot cast here as that will lower the scale.
                     let s = Int128Chunked::from_chunks(name, chunks)
-                        .into_decimal_unchecked(Some(*precision), *scale)
+                        .into_decimal_unchecked(*precision, *scale)
                         .into_series();
                     Ok(s)
                 })
             },
-            ArrowDataType::Decimal(precision, scale)
-            | ArrowDataType::Decimal256(precision, scale) => {
+            ArrowDataType::Decimal(precision, scale) => {
                 feature_gated!("dtype-decimal", {
-                    polars_ensure!(*scale <= *precision, InvalidOperation: "invalid decimal precision and scale (prec={precision}, scale={scale})");
-                    polars_ensure!(*precision <= 38, InvalidOperation: "polars does not support decimals above 38 precision");
+                    polars_compute::decimal::dec128_verify_prec_scale(*precision, *scale)?;
 
-                    // Q? I don't think this is correct for Decimal256?
                     let mut chunks = chunks;
                     for chunk in chunks.iter_mut() {
                         *chunk = std::mem::take(
@@ -375,9 +373,43 @@ impl Series {
                         .to_boxed();
                     }
 
-                    // @NOTE: We cannot cast here as that will lower the scale.
                     let s = Int128Chunked::from_chunks(name, chunks)
-                        .into_decimal_unchecked(Some(*precision), *scale)
+                        .into_decimal_unchecked(*precision, *scale)
+                        .into_series();
+                    Ok(s)
+                })
+            },
+            ArrowDataType::Decimal256(precision, scale) => {
+                feature_gated!("dtype-decimal", {
+                    use arrow::types::i256;
+
+                    polars_compute::decimal::dec128_verify_prec_scale(*precision, *scale)?;
+
+                    let mut chunks = chunks;
+                    for chunk in chunks.iter_mut() {
+                        let arr = std::mem::take(
+                            chunk
+                                .as_any_mut()
+                                .downcast_mut::<PrimitiveArray<i256>>()
+                                .unwrap(),
+                        );
+                        let arr_128: PrimitiveArray<i128> = arr.iter().map(|opt_v| {
+                            if let Some(v) = opt_v {
+                                let smaller: Option<i128> = (*v).try_into().ok();
+                                let smaller = smaller.filter(|v| dec128_fits(*v, *precision));
+                                smaller.ok_or_else(|| {
+                                    polars_err!(ComputeError: "Decimal256 to Decimal128 conversion overflowed, Decimal256 is not (yet) supported in Polars")
+                                }).map(Some)
+                            } else {
+                                Ok(None)
+                            }
+                        }).try_collect_arr_trusted()?;
+
+                        *chunk = arr_128.to(ArrowDataType::Int128).to_boxed();
+                    }
+
+                    let s = Int128Chunked::from_chunks(name, chunks)
+                        .into_decimal_unchecked(*precision, *scale)
                         .into_series();
                     Ok(s)
                 })
