@@ -1,3 +1,8 @@
+#![cfg_attr(
+    feature = "allow_unused",
+    allow(unused, dead_code, irrefutable_let_patterns)
+)] // Maybe be caused by some feature
+pub mod dispatch;
 mod expressions;
 pub mod groups;
 pub mod hash_keys;
