@@ -24,12 +24,11 @@ use object_store::{BackoffConfig, RetryConfig};
 use polars_error::*;
 #[cfg(feature = "aws")]
 use polars_utils::cache::LruCache;
+use polars_utils::plpath::{CloudScheme, PlPathRef};
 #[cfg(feature = "http")]
 use reqwest::header::HeaderMap;
 #[cfg(feature = "serde")]
 use serde::{Deserialize, Serialize};
-#[cfg(feature = "cloud")]
-use url::Url;
 
 #[cfg(feature = "cloud")]
 use super::credential_provider::PlCredentialProvider;
@@ -153,62 +152,35 @@ pub enum CloudType {
     Aws,
     Azure,
     File,
+    /// Google cloud platform
     Gcp,
     Http,
+    /// HuggingFace
     Hf,
 }
 
 impl CloudType {
-    #[cfg(feature = "cloud")]
-    pub(crate) fn from_url(parsed: &Url) -> PolarsResult<Self> {
-        Ok(match parsed.scheme() {
-            "s3" | "s3a" => Self::Aws,
-            "az" | "azure" | "adl" | "abfs" | "abfss" => Self::Azure,
-            "gs" | "gcp" | "gcs" => Self::Gcp,
-            "file" => Self::File,
-            "http" | "https" => Self::Http,
-            "hf" => Self::Hf,
-            _ => polars_bail!(ComputeError: "unknown url scheme"),
-        })
+    pub fn from_cloud_scheme(scheme: &CloudScheme) -> Self {
+        match scheme {
+            CloudScheme::Abfs
+            | CloudScheme::Abfss
+            | CloudScheme::Adl
+            | CloudScheme::Az
+            | CloudScheme::Azure => Self::Azure,
+
+            CloudScheme::File | CloudScheme::FileNoHostname => Self::File,
+
+            CloudScheme::Gcs | CloudScheme::Gs => Self::Gcp,
+
+            CloudScheme::Hf => Self::Hf,
+
+            CloudScheme::Http | CloudScheme::Https => Self::Http,
+
+            CloudScheme::S3 | CloudScheme::S3a => Self::Aws,
+        }
     }
 }
 
-#[cfg(feature = "cloud")]
-pub(crate) fn parse_url(input: &str) -> std::result::Result<url::Url, url::ParseError> {
-    Ok(if input.contains("://") {
-        if input.starts_with("http://") || input.starts_with("https://") {
-            url::Url::parse(input)
-        } else {
-            url::Url::parse(&input.replace("%", "%25"))
-        }?
-    } else {
-        let path = std::path::Path::new(input);
-        let mut tmp;
-        url::Url::from_file_path(if path.is_relative() {
-            tmp = std::env::current_dir().unwrap();
-            tmp.push(path);
-            tmp.as_path()
-        } else {
-            path
-        })
-        .unwrap()
-    })
-}
-
-impl FromStr for CloudType {
-    type Err = PolarsError;
-
-    #[cfg(feature = "cloud")]
-    fn from_str(url: &str) -> Result<Self, Self::Err> {
-        let parsed = parse_url(url).map_err(to_compute_err)?;
-        Self::from_url(&parsed)
-    }
-
-    #[cfg(not(feature = "cloud"))]
-    fn from_str(_s: &str) -> Result<Self, Self::Err> {
-        polars_bail!(ComputeError: "at least one of the cloud features must be enabled");
-    }
-}
 #[cfg(any(feature = "aws", feature = "gcp", feature = "azure"))]
 fn get_retry_config(max_retries: usize) -> RetryConfig {
     RetryConfig {
@@ -218,8 +190,12 @@ fn get_retry_config(max_retries: usize) -> RetryConfig {
     }
 }
 
+pub static USER_AGENT: &str = concat!("polars", "/", env!("CARGO_PKG_VERSION"),);
+
 #[cfg(any(feature = "aws", feature = "gcp", feature = "azure", feature = "http"))]
 pub(super) fn get_client_options() -> ClientOptions {
+    use reqwest::header::HeaderValue;
+
     ClientOptions::new()
         // We set request timeout super high as the timeout isn't reset at ACK,
         // but starts from the moment we start downloading a body.
@@ -227,6 +203,7 @@ pub(super) fn get_client_options() -> ClientOptions {
         .with_timeout_disabled()
         // Concurrency can increase connection latency, so set to None, similar to default.
         .with_connect_timeout_disabled()
+        .with_user_agent(HeaderValue::from_static(USER_AGENT))
         .with_allow_http(true)
 }
 
@@ -293,10 +270,15 @@ impl CloudOptions {
 
     /// Build the [`object_store::ObjectStore`] implementation for AWS.
     #[cfg(feature = "aws")]
-    pub async fn build_aws(&self, url: &str) -> PolarsResult<impl object_store::ObjectStore> {
+    pub async fn build_aws(
+        &self,
+        url: &str,
+        clear_cached_credentials: bool,
+    ) -> PolarsResult<impl object_store::ObjectStore> {
         use super::credential_provider::IntoCredentialProvider;
 
-        let opt_credential_provider = self.initialized_credential_provider()?;
+        let opt_credential_provider =
+            self.initialized_credential_provider(clear_cached_credentials)?;
 
         let mut builder = AmazonS3Builder::from_env()
             .with_client_options(get_client_options())
@@ -360,7 +342,7 @@ impl CloudOptions {
                 .get_config_value(&AmazonS3ConfigKey::Region)
                 .is_none()
         {
-            let bucket = crate::cloud::CloudLocation::new(url, false)?.bucket;
+            let bucket = crate::cloud::CloudLocation::new(PlPathRef::new(url), false)?.bucket;
             let region = {
                 let mut bucket_region = BUCKET_REGION.lock().unwrap();
                 bucket_region.get(bucket.as_str()).cloned()
@@ -384,6 +366,7 @@ impl CloudOptions {
                         );
                         let result = with_concurrency_budget(1, || async {
                             reqwest::Client::builder()
+                                .user_agent(USER_AGENT)
                                 .build()
                                 .unwrap()
                                 .head(format!("https://{bucket}.s3.amazonaws.com"))
@@ -409,7 +392,7 @@ impl CloudOptions {
         let opt_credential_provider = match opt_credential_provider {
             #[cfg(feature = "python")]
             Some(PlCredentialProvider::Python(object)) => {
-                if pyo3::Python::with_gil(|py| {
+                if pyo3::Python::attach(|py| {
                     let Ok(func_object) = object
                         .unwrap_as_provider_ref()
                         .getattr(py, "_can_use_as_provider")
@@ -453,7 +436,11 @@ impl CloudOptions {
 
     /// Build the [`object_store::ObjectStore`] implementation for Azure.
     #[cfg(feature = "azure")]
-    pub fn build_azure(&self, url: &str) -> PolarsResult<impl object_store::ObjectStore> {
+    pub fn build_azure(
+        &self,
+        url: &str,
+        clear_cached_credentials: bool,
+    ) -> PolarsResult<impl object_store::ObjectStore> {
         use super::credential_provider::IntoCredentialProvider;
 
         let verbose = polars_core::config::verbose();
@@ -476,17 +463,18 @@ impl CloudOptions {
             .with_url(url)
             .with_retry(get_retry_config(self.max_retries));
 
-        let builder = if let Some(v) = self.initialized_credential_provider()? {
-            if verbose {
-                eprintln!(
-                    "[CloudOptions::build_azure]: Using credential provider {:?}",
-                    &v
-                );
-            }
-            builder.with_credentials(v.into_azure_provider())
-        } else {
-            builder
-        };
+        let builder =
+            if let Some(v) = self.initialized_credential_provider(clear_cached_credentials)? {
+                if verbose {
+                    eprintln!(
+                        "[CloudOptions::build_azure]: Using credential provider {:?}",
+                        &v
+                    );
+                }
+                builder.with_credentials(v.into_azure_provider())
+            } else {
+                builder
+            };
 
         let out = builder.build()?;
 
@@ -507,10 +495,14 @@ impl CloudOptions {
 
     /// Build the [`object_store::ObjectStore`] implementation for GCP.
     #[cfg(feature = "gcp")]
-    pub fn build_gcp(&self, url: &str) -> PolarsResult<impl object_store::ObjectStore> {
+    pub fn build_gcp(
+        &self,
+        url: &str,
+        clear_cached_credentials: bool,
+    ) -> PolarsResult<impl object_store::ObjectStore> {
         use super::credential_provider::IntoCredentialProvider;
 
-        let credential_provider = self.initialized_credential_provider()?;
+        let credential_provider = self.initialized_credential_provider(clear_cached_credentials)?;
 
         let builder = if credential_provider.is_none() {
             GoogleCloudStorageBuilder::from_env()
@@ -533,7 +525,7 @@ impl CloudOptions {
             .with_url(url)
             .with_retry(get_retry_config(self.max_retries));
 
-        let builder = if let Some(v) = credential_provider.clone() {
+        let builder = if let Some(v) = credential_provider {
             builder.with_credentials(v.into_gcp_provider())
         } else {
             builder
@@ -565,10 +557,10 @@ impl CloudOptions {
     /// Parse a configuration from a Hashmap. This is the interface from Python.
     #[allow(unused_variables)]
     pub fn from_untyped_config<I: IntoIterator<Item = (impl AsRef<str>, impl Into<String>)>>(
-        url: &str,
+        scheme: Option<&CloudScheme>,
         config: I,
     ) -> PolarsResult<Self> {
-        match CloudType::from_str(url)? {
+        match scheme.map_or(CloudType::File, CloudType::from_cloud_scheme) {
             CloudType::Aws => {
                 #[cfg(feature = "aws")]
                 {
@@ -678,9 +670,12 @@ impl CloudOptions {
     /// Python passes a credential provider builder that needs to be called to get the actual credential
     /// provider.
     #[cfg(feature = "cloud")]
-    fn initialized_credential_provider(&self) -> PolarsResult<Option<PlCredentialProvider>> {
+    fn initialized_credential_provider(
+        &self,
+        clear_cached_credentials: bool,
+    ) -> PolarsResult<Option<PlCredentialProvider>> {
         if let Some(v) = self.credential_provider.clone() {
-            v.try_into_initialized()
+            v.try_into_initialized(clear_cached_credentials)
         } else {
             Ok(None)
         }
@@ -692,81 +687,8 @@ impl CloudOptions {
 mod tests {
     use hashbrown::HashMap;
 
-    use super::{parse_untyped_config, parse_url};
+    use super::parse_untyped_config;
 
-    #[test]
-    fn test_parse_url() {
-        assert_eq!(
-            parse_url(r"http://Users/Jane Doe/data.csv")
-                .unwrap()
-                .as_str(),
-            "http://users/Jane%20Doe/data.csv"
-        );
-        assert_eq!(
-            parse_url(r"http://Users/Jane Doe/data.csv")
-                .unwrap()
-                .as_str(),
-            "http://users/Jane%20Doe/data.csv"
-        );
-        #[cfg(target_os = "windows")]
-        {
-            assert_eq!(
-                parse_url(r"file:///c:/Users/Jane Doe/data.csv")
-                    .unwrap()
-                    .as_str(),
-                "file:///c:/Users/Jane%20Doe/data.csv"
-            );
-            assert_eq!(
-                parse_url(r"file://\c:\Users\Jane Doe\data.csv")
-                    .unwrap()
-                    .as_str(),
-                "file:///c:/Users/Jane%20Doe/data.csv"
-            );
-            assert_eq!(
-                parse_url(r"c:\Users\Jane Doe\data.csv").unwrap().as_str(),
-                "file:///C:/Users/Jane%20Doe/data.csv"
-            );
-            assert_eq!(
-                parse_url(r"data.csv").unwrap().as_str(),
-                url::Url::from_file_path(
-                    [
-                        std::env::current_dir().unwrap().as_path(),
-                        std::path::Path::new("data.csv")
-                    ]
-                    .into_iter()
-                    .collect::<std::path::PathBuf>()
-                )
-                .unwrap()
-                .as_str()
-            );
-        }
-        #[cfg(not(target_os = "windows"))]
-        {
-            assert_eq!(
-                parse_url(r"file:///home/Jane Doe/data.csv")
-                    .unwrap()
-                    .as_str(),
-                "file:///home/Jane%20Doe/data.csv"
-            );
-            assert_eq!(
-                parse_url(r"/home/Jane Doe/data.csv").unwrap().as_str(),
-                "file:///home/Jane%20Doe/data.csv"
-            );
-            assert_eq!(
-                parse_url(r"data.csv").unwrap().as_str(),
-                url::Url::from_file_path(
-                    [
-                        std::env::current_dir().unwrap().as_path(),
-                        std::path::Path::new("data.csv")
-                    ]
-                    .into_iter()
-                    .collect::<std::path::PathBuf>()
-                )
-                .unwrap()
-                .as_str()
-            );
-        }
-    }
     #[cfg(feature = "aws")]
     #[test]
     fn test_parse_untyped_config() {
