@@ -1,10 +1,10 @@
-use std::path::{Path, PathBuf};
-
+use arrow::buffer::Buffer;
 use polars_core::prelude::*;
 use polars_io::cloud::CloudOptions;
 use polars_io::parquet::read::ParallelStrategy;
 use polars_io::prelude::ParquetOptions;
 use polars_io::{HiveOptions, RowIndex};
+use polars_utils::plpath::PlPath;
 use polars_utils::slice_enum::Slice;
 
 use crate::prelude::*;
@@ -79,7 +79,10 @@ impl LazyFileListReader for LazyParquetReader {
             rechunk: self.args.rechunk,
             cache: self.args.cache,
             glob: self.args.glob,
+            hidden_file_prefix: None,
             projection: None,
+            column_mapping: None,
+            default_values: None,
             // Note: We call `with_row_index()` on the LazyFrame below
             row_index: None,
             pre_slice: self
@@ -94,7 +97,9 @@ impl LazyFileListReader for LazyParquetReader {
             },
             extra_columns_policy: ExtraColumnsPolicy::Raise,
             include_file_paths: self.args.include_file_paths,
-            deletion_files: Default::default(),
+            deletion_files: None,
+            table_statistics: None,
+            row_count: None,
         };
 
         let mut lf: LazyFrame =
@@ -161,11 +166,8 @@ impl LazyFileListReader for LazyParquetReader {
 
 impl LazyFrame {
     /// Create a LazyFrame directly from a parquet scan.
-    pub fn scan_parquet(path: impl AsRef<Path>, args: ScanArgsParquet) -> PolarsResult<Self> {
-        Self::scan_parquet_sources(
-            ScanSources::Paths([path.as_ref().to_path_buf()].into()),
-            args,
-        )
+    pub fn scan_parquet(path: PlPath, args: ScanArgsParquet) -> PolarsResult<Self> {
+        Self::scan_parquet_sources(ScanSources::Paths(Buffer::from_iter([path])), args)
     }
 
     /// Create a LazyFrame directly from a parquet scan.
@@ -174,7 +176,7 @@ impl LazyFrame {
     }
 
     /// Create a LazyFrame directly from a parquet scan.
-    pub fn scan_parquet_files(paths: Arc<[PathBuf]>, args: ScanArgsParquet) -> PolarsResult<Self> {
+    pub fn scan_parquet_files(paths: Buffer<PlPath>, args: ScanArgsParquet) -> PolarsResult<Self> {
         Self::scan_parquet_sources(ScanSources::Paths(paths), args)
     }
 }
