@@ -43,39 +43,15 @@ fn take_right(total_rows: IdxSize, n_rows_right: IdxSize, slice: Option<(i64, us
 }
 
 pub trait CrossJoin: IntoDf {
-    #[doc(hidden)]
-    /// used by streaming
-    fn _cross_join_with_names(
-        &self,
-        other: &DataFrame,
-        names: &[PlSmallStr],
-    ) -> PolarsResult<DataFrame> {
-        let (mut l_df, r_df) = cross_join_dfs(self.to_df(), other, None, false)?;
-        l_df.clear_schema();
-
-        unsafe {
-            l_df.get_columns_mut().extend_from_slice(r_df.get_columns());
-
-            l_df.get_columns_mut()
-                .iter_mut()
-                .zip(names)
-                .for_each(|(s, name)| {
-                    if s.name() != name {
-                        s.rename(name.clone());
-                    }
-                });
-        }
-        Ok(l_df)
-    }
-
     /// Creates the Cartesian product from both frames, preserves the order of the left keys.
     fn cross_join(
         &self,
         other: &DataFrame,
         suffix: Option<PlSmallStr>,
         slice: Option<(i64, usize)>,
+        maintain_order: MaintainOrderJoin,
     ) -> PolarsResult<DataFrame> {
-        let (l_df, r_df) = cross_join_dfs(self.to_df(), other, slice, true)?;
+        let (l_df, r_df) = cross_join_dfs(self.to_df(), other, slice, true, maintain_order)?;
 
         _finish_join(l_df, r_df, suffix)
     }
@@ -83,12 +59,27 @@ pub trait CrossJoin: IntoDf {
 
 impl CrossJoin for DataFrame {}
 
-fn cross_join_dfs(
-    df_self: &DataFrame,
-    other: &DataFrame,
+fn cross_join_dfs<'a>(
+    mut df_self: &'a DataFrame,
+    mut other: &'a DataFrame,
     slice: Option<(i64, usize)>,
     parallel: bool,
+    maintain_order: MaintainOrderJoin,
 ) -> PolarsResult<(DataFrame, DataFrame)> {
+    if df_self.height() == 0 || other.height() == 0 {
+        return Ok((df_self.clear(), other.clear()));
+    }
+
+    let left_is_primary = match maintain_order {
+        MaintainOrderJoin::None => true,
+        MaintainOrderJoin::Left | MaintainOrderJoin::LeftRight => true,
+        MaintainOrderJoin::Right | MaintainOrderJoin::RightLeft => false,
+    };
+
+    if !left_is_primary {
+        core::mem::swap(&mut df_self, &mut other);
+    }
+
     let n_rows_left = df_self.height() as IdxSize;
     let n_rows_right = other.height() as IdxSize;
     let Some(total_rows) = n_rows_left.checked_mul(n_rows_right) else {
@@ -97,9 +88,6 @@ fn cross_join_dfs(
             consider compiling with polars-big-idx feature, or set 'streaming'"
         );
     };
-    if n_rows_left == 0 || n_rows_right == 0 {
-        return Ok((df_self.clear(), other.clear()));
-    }
 
     // the left side has the Nth row combined with every row from right.
     // So let's say we have the following no. of rows
@@ -138,7 +126,11 @@ fn cross_join_dfs(
     } else {
         (create_left_df(), create_right_df())
     };
-    Ok((l_df, r_df))
+    if left_is_primary {
+        Ok((l_df, r_df))
+    } else {
+        Ok((r_df, l_df))
+    }
 }
 
 pub(super) fn fused_cross_filter(
@@ -146,18 +138,26 @@ pub(super) fn fused_cross_filter(
     right: &DataFrame,
     suffix: Option<PlSmallStr>,
     cross_join_options: &CrossJoinOptions,
+    maintain_order: MaintainOrderJoin,
 ) -> PolarsResult<DataFrame> {
-    // Because we do a cartesian product, the number of partitions is squared.
-    // We take the sqrt, but we don't expect every partition to produce results and work can be
-    // imbalanced, so we multiply the number of partitions by 2;
-    let n_partitions = (_set_partition_size() as f32).sqrt() as usize * 2;
-    let splitted_a = split(left, n_partitions);
-    let splitted_b = split(right, n_partitions);
+    let unfiltered_size = (left.height() as u64).saturating_mul(right.height() as u64);
+    let chunk_size = (unfiltered_size / _set_partition_size() as u64).clamp(1, 100_000);
+    let num_chunks = (unfiltered_size / chunk_size).max(1) as usize;
 
-    let cartesian_prod = splitted_a
-        .iter()
-        .flat_map(|l| splitted_b.iter().map(move |r| (l, r)))
-        .collect::<Vec<_>>();
+    let left_is_primary = match maintain_order {
+        MaintainOrderJoin::None => true,
+        MaintainOrderJoin::Left | MaintainOrderJoin::LeftRight => true,
+        MaintainOrderJoin::Right | MaintainOrderJoin::RightLeft => false,
+    };
+
+    let split_chunks;
+    let cartesian_prod = if left_is_primary {
+        split_chunks = split(left, num_chunks);
+        split_chunks.iter().map(|l| (l, right)).collect::<Vec<_>>()
+    } else {
+        split_chunks = split(right, num_chunks);
+        split_chunks.iter().map(|r| (left, r)).collect::<Vec<_>>()
+    };
 
     let names = _finish_join(left.clear(), right.clear(), suffix)?;
     let rename_names = names.get_column_names();
@@ -166,7 +166,7 @@ pub(super) fn fused_cross_filter(
     let dfs = POOL
         .install(|| {
             cartesian_prod.par_iter().map(|(left, right)| {
-                let (mut left, right) = cross_join_dfs(left, right, None, false)?;
+                let (mut left, right) = cross_join_dfs(left, right, None, false, maintain_order)?;
                 let mut right_columns = right.take_columns();
 
                 for (c, name) in right_columns.iter_mut().zip(rename_names) {
