@@ -46,8 +46,8 @@ pub fn business_day_count(
 
     let out = match (start_dates.len(), end_dates.len()) {
         (_, 1) => {
-            if let Some(end_date) = end_dates.get(0) {
-                start_dates.apply_values(|start_date| {
+            if let Some(end_date) = end_dates.physical().get(0) {
+                start_dates.physical().apply_values(|start_date| {
                     business_day_count_impl(
                         start_date,
                         end_date,
@@ -61,8 +61,8 @@ pub fn business_day_count(
             }
         },
         (1, _) => {
-            if let Some(start_date) = start_dates.get(0) {
-                end_dates.apply_values(|end_date| {
+            if let Some(start_date) = start_dates.physical().get(0) {
+                end_dates.physical().apply_values(|end_date| {
                     business_day_count_impl(
                         start_date,
                         end_date,
@@ -82,17 +82,22 @@ pub fn business_day_count(
                 start_dates.len(),
                 end_dates.len()
             );
-            binary_elementwise_values(start_dates, end_dates, |start_date, end_date| {
-                business_day_count_impl(
-                    start_date,
-                    end_date,
-                    &week_mask,
-                    n_business_days_in_week_mask,
-                    &holidays,
-                )
-            })
+            binary_elementwise_values(
+                start_dates.physical(),
+                end_dates.physical(),
+                |start_date, end_date| {
+                    business_day_count_impl(
+                        start_date,
+                        end_date,
+                        &week_mask,
+                        n_business_days_in_week_mask,
+                        &holidays,
+                    )
+                },
+            )
         },
     };
+    let out = out.with_name(start_dates.name().clone());
     Ok(out.into_series())
 }
 
@@ -217,24 +222,26 @@ pub fn add_business_days(
     let out: Int32Chunked = match (start_dates.len(), n.len()) {
         (_, 1) => {
             if let Some(n) = n.get(0) {
-                start_dates.try_apply_nonnull_values_generic(|start_date| {
-                    let (start_date, day_of_week) =
-                        roll_start_date(start_date, roll, &week_mask, &holidays)?;
-                    Ok::<i32, PolarsError>(add_business_days_impl(
-                        start_date,
-                        day_of_week,
-                        n,
-                        &week_mask,
-                        n_business_days_in_week_mask,
-                        &holidays,
-                    ))
-                })?
+                start_dates
+                    .physical()
+                    .try_apply_nonnull_values_generic(|start_date| {
+                        let (start_date, day_of_week) =
+                            roll_start_date(start_date, roll, &week_mask, &holidays)?;
+                        Ok::<i32, PolarsError>(add_business_days_impl(
+                            start_date,
+                            day_of_week,
+                            n,
+                            &week_mask,
+                            n_business_days_in_week_mask,
+                            &holidays,
+                        ))
+                    })?
             } else {
                 Int32Chunked::full_null(start_dates.name().clone(), start_dates.len())
             }
         },
         (1, _) => {
-            if let Some(start_date) = start_dates.get(0) {
+            if let Some(start_date) = start_dates.physical().get(0) {
                 let (start_date, day_of_week) =
                     roll_start_date(start_date, roll, &week_mask, &holidays)?;
                 n.apply_values(|n| {
@@ -258,7 +265,7 @@ pub fn add_business_days(
                 start_dates.len(),
                 n.len()
             );
-            try_binary_elementwise(start_dates, n, |opt_start_date, opt_n| {
+            try_binary_elementwise(start_dates.physical(), n, |opt_start_date, opt_n| {
                 match (opt_start_date, opt_n) {
                     (Some(start_date), Some(n)) => {
                         let (start_date, day_of_week) =
@@ -368,11 +375,17 @@ pub fn is_business_day(
     // Sort now so we can use `binary_search` in the hot for-loop.
     let holidays = normalise_holidays(holidays, &week_mask);
     let dates = dates.date()?;
-    let out: BooleanChunked = dates.apply_nonnull_values_generic(DataType::Boolean, |date| {
-        let day_of_week = get_day_of_week(date);
-        // SAFETY: week_mask is length 7, day_of_week is between 0 and 6
-        unsafe { (*week_mask.get_unchecked(day_of_week)) && holidays.binary_search(&date).is_err() }
-    });
+    let out: BooleanChunked =
+        dates
+            .physical()
+            .apply_nonnull_values_generic(DataType::Boolean, |date| {
+                let day_of_week = get_day_of_week(date);
+                // SAFETY: week_mask is length 7, day_of_week is between 0 and 6
+                unsafe {
+                    (*week_mask.get_unchecked(day_of_week))
+                        && holidays.binary_search(&date).is_err()
+                }
+            });
     Ok(out.into_series())
 }
 
