@@ -11,6 +11,7 @@ mod cum_agg;
 mod cut;
 #[cfg(feature = "diff")]
 mod diff;
+mod eager;
 #[cfg(feature = "ewma")]
 mod ewm;
 #[cfg(feature = "ewma_by")]
@@ -28,6 +29,8 @@ mod int_range;
 mod interpolation;
 #[cfg(feature = "is_between")]
 mod is_between;
+#[cfg(feature = "is_close")]
+mod is_close;
 #[cfg(feature = "is_first_distinct")]
 mod is_first_distinct;
 #[cfg(feature = "is_in")]
@@ -58,6 +61,7 @@ mod rolling;
 pub mod round;
 #[cfg(feature = "search_sorted")]
 mod search_sorted;
+mod strings;
 #[cfg(feature = "to_dummies")]
 mod to_dummies;
 #[cfg(feature = "unique_counts")]
@@ -77,6 +81,7 @@ pub use cum_agg::*;
 pub use cut::*;
 #[cfg(feature = "diff")]
 pub use diff::*;
+pub use eager::*;
 #[cfg(feature = "ewma")]
 pub use ewm::*;
 #[cfg(feature = "ewma_by")]
@@ -98,6 +103,8 @@ pub use interpolation::interpolate_by::*;
 pub use interpolation::*;
 #[cfg(feature = "is_between")]
 pub use is_between::*;
+#[cfg(feature = "is_close")]
+pub use is_close::*;
 #[cfg(feature = "is_first_distinct")]
 pub use is_first_distinct::*;
 #[cfg(feature = "is_in")]
@@ -130,6 +137,7 @@ pub use rolling::*;
 pub use round::*;
 #[cfg(feature = "search_sorted")]
 pub use search_sorted::*;
+pub use strings::*;
 #[cfg(feature = "to_dummies")]
 pub use to_dummies::*;
 #[cfg(feature = "unique_counts")]
