@@ -1,12 +1,12 @@
 use polars_utils::pl_str::PlSmallStr;
-#[cfg(feature = "serde_types")]
+#[cfg(feature = "serde")]
 use serde::{Deserialize, Serialize};
 
 use super::super::Repetition;
 
 /// Common type information.
 #[derive(Clone, Debug, PartialEq, Eq, Hash)]
-#[cfg_attr(feature = "serde_types", derive(Deserialize, Serialize))]
+#[cfg_attr(feature = "serde", derive(Deserialize, Serialize))]
 pub struct FieldInfo {
     /// The field name
     pub name: PlSmallStr,
