@@ -9,7 +9,7 @@ use polars_utils::format_pl_smallstr;
 use serde::{Deserialize, Serialize};
 
 use super::*;
-use crate::constants::{get_len_name, get_literal_name};
+use crate::constants::{PL_ELEMENT_NAME, get_len_name, get_literal_name};
 
 #[derive(Default, Debug, Clone, Hash, PartialEq, Eq)]
 #[cfg_attr(feature = "ir_serde", derive(Serialize, Deserialize))]
@@ -60,7 +60,7 @@ impl OutputName {
     }
 }
 
-#[derive(Debug)]
+#[derive(Clone, Debug)]
 #[cfg_attr(feature = "ir_serde", derive(Serialize, Deserialize))]
 pub struct ExprIR {
     /// Output name of this expression.
@@ -81,22 +81,13 @@ impl PartialEq for ExprIR {
     }
 }
 
-impl Clone for ExprIR {
-    fn clone(&self) -> Self {
-        let output_dtype = OnceLock::new();
-        if let Some(dt) = self.output_dtype.get() {
-            output_dtype.set(dt.clone()).unwrap()
-        }
-
-        ExprIR {
-            output_name: self.output_name.clone(),
-            node: self.node,
-            output_dtype,
-        }
+impl Borrow<Node> for ExprIR {
+    fn borrow(&self) -> &Node {
+        &self.node
     }
 }
 
-impl Borrow<Node> for ExprIR {
+impl Borrow<Node> for &ExprIR {
     fn borrow(&self) -> &Node {
         &self.node
     }
@@ -130,6 +121,10 @@ impl ExprIR {
         out.node = node;
         for (_, ae) in arena.iter(node) {
             match ae {
+                AExpr::Element => {
+                    out.output_name = OutputName::ColumnLhs(PL_ELEMENT_NAME.clone());
+                    break;
+                },
                 AExpr::Column(name) => {
                     out.output_name = OutputName::ColumnLhs(name.clone());
                     break;
@@ -268,31 +263,25 @@ impl ExprIR {
         is_scalar_ae(self.node, expr_arena)
     }
 
-    pub fn dtype(
-        &self,
-        schema: &Schema,
-        ctxt: Context,
-        expr_arena: &Arena<AExpr>,
-    ) -> PolarsResult<&DataType> {
+    pub fn is_length_preserving(&self, expr_arena: &Arena<AExpr>) -> bool {
+        is_length_preserving_ae(self.node, expr_arena)
+    }
+
+    pub fn dtype(&self, schema: &Schema, expr_arena: &Arena<AExpr>) -> PolarsResult<&DataType> {
         match self.output_dtype.get() {
             Some(dtype) => Ok(dtype),
             None => {
                 let dtype = expr_arena
                     .get(self.node)
-                    .to_dtype(schema, ctxt, expr_arena)?;
+                    .to_dtype(&ToFieldContext::new(expr_arena, schema))?;
                 let _ = self.output_dtype.set(dtype);
                 Ok(self.output_dtype.get().unwrap())
             },
         }
     }
 
-    pub fn field(
-        &self,
-        schema: &Schema,
-        ctxt: Context,
-        expr_arena: &Arena<AExpr>,
-    ) -> PolarsResult<Field> {
-        let dtype = self.dtype(schema, ctxt, expr_arena)?;
+    pub fn field(&self, schema: &Schema, expr_arena: &Arena<AExpr>) -> PolarsResult<Field> {
+        let dtype = self.dtype(schema, expr_arena)?;
         let name = self.output_name();
         Ok(Field::new(name.clone(), dtype.clone()))
     }
