@@ -1,11 +1,13 @@
 use std::fmt;
 use std::path::PathBuf;
 
+use polars_core::prelude::{InitHashMaps, PlHashSet};
 use polars_core::schema::Schema;
 use polars_utils::pl_str::PlSmallStr;
+use polars_utils::unique_id::UniqueId;
+use recursive::recursive;
 
 use super::format::ExprIRSliceDisplay;
-use crate::constants::UNLIMITED_CACHE;
 use crate::prelude::ir::format::ColumnsDisplay;
 use crate::prelude::*;
 
@@ -18,14 +20,14 @@ const INDENT: &str = "  ";
 #[derive(Clone, Copy)]
 enum DotNode {
     Plain(usize),
-    Cache(usize),
+    Cache(UniqueId),
 }
 
 impl fmt::Display for DotNode {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             DotNode::Plain(n) => write!(f, "p{n}"),
-            DotNode::Cache(n) => write!(f, "c{n}"),
+            DotNode::Cache(n) => write!(f, "\"{n}\""),
         }
     }
 }
@@ -69,55 +71,66 @@ impl<'a> IRDotDisplay<'a> {
         }
     }
 
+    #[recursive]
     fn _format(
         &self,
         f: &mut fmt::Formatter<'_>,
         parent: Option<DotNode>,
         last: &mut usize,
+        visited_caches: &mut PlHashSet<UniqueId>,
     ) -> std::fmt::Result {
         use fmt::Write;
 
         let root = self.lp.root();
         let id = if let IR::Cache { id, .. } = root {
-            DotNode::Cache(id.to_usize())
+            DotNode::Cache(*id)
         } else {
             *last += 1;
             DotNode::Plain(*last)
         };
 
         if let Some(parent) = parent {
-            writeln!(f, "{INDENT}{parent} -- {id}")?;
+            writeln!(f, "{INDENT}{id} -> {parent}")?;
+        }
+
+        macro_rules! recurse {
+            ($input:expr) => {
+                self.with_root($input)
+                    ._format(f, Some(id), last, visited_caches)?;
+            };
         }
 
         use IR::*;
         match root {
             Union { inputs, .. } => {
                 for input in inputs {
-                    self.with_root(*input)._format(f, Some(id), last)?;
+                    recurse!(*input);
                 }
 
                 write_label(f, id, |f| f.write_str("UNION"))?;
             },
             HConcat { inputs, .. } => {
                 for input in inputs {
-                    self.with_root(*input)._format(f, Some(id), last)?;
+                    recurse!(*input);
                 }
 
                 write_label(f, id, |f| f.write_str("HCONCAT"))?;
             },
             Cache {
-                input, cache_hits, ..
+                input,
+                id: cache_id,
+                ..
             } => {
-                self.with_root(*input)._format(f, Some(id), last)?;
+                if !visited_caches.contains(cache_id) {
+                    visited_caches.insert(*cache_id);
+
+                    recurse!(*input);
 
-                if *cache_hits == UNLIMITED_CACHE {
                     write_label(f, id, |f| f.write_str("CACHE"))?;
-                } else {
-                    write_label(f, id, |f| write!(f, "CACHE: {cache_hits} times"))?;
-                };
+                }
             },
             Filter { predicate, input } => {
-                self.with_root(*input)._format(f, Some(id), last)?;
+                recurse!(*input);
 
                 let pred = self.display_expr(predicate);
                 write_label(f, id, |f| write!(f, "FILTER BY {pred}"))?;
@@ -145,14 +158,14 @@ impl<'a> IRDotDisplay<'a> {
                 schema,
                 ..
             } => {
-                self.with_root(*input)._format(f, Some(id), last)?;
+                recurse!(*input);
                 write_label(f, id, |f| write!(f, "π {}/{}", expr.len(), schema.len()))?;
             },
             Sort {
                 input, by_column, ..
             } => {
                 let by_column = self.display_exprs(by_column);
-                self.with_root(*input)._format(f, Some(id), last)?;
+                recurse!(*input);
                 write_label(f, id, |f| write!(f, "SORT BY {by_column}"))?;
             },
             GroupBy {
@@ -160,20 +173,20 @@ impl<'a> IRDotDisplay<'a> {
             } => {
                 let keys = self.display_exprs(keys);
                 let aggs = self.display_exprs(aggs);
-                self.with_root(*input)._format(f, Some(id), last)?;
+                recurse!(*input);
                 write_label(f, id, |f| write!(f, "AGG {aggs}\nBY\n{keys}"))?;
             },
             HStack { input, exprs, .. } => {
                 let exprs = self.display_exprs(exprs);
-                self.with_root(*input)._format(f, Some(id), last)?;
+                recurse!(*input);
                 write_label(f, id, |f| write!(f, "WITH COLUMNS {exprs}"))?;
             },
             Slice { input, offset, len } => {
-                self.with_root(*input)._format(f, Some(id), last)?;
+                recurse!(*input);
                 write_label(f, id, |f| write!(f, "SLICE offset: {offset}; len: {len}"))?;
             },
             Distinct { input, options, .. } => {
-                self.with_root(*input)._format(f, Some(id), last)?;
+                recurse!(*input);
                 write_label(f, id, |f| {
                     f.write_str("DISTINCT")?;
 
@@ -212,10 +225,10 @@ impl<'a> IRDotDisplay<'a> {
                 file_info,
                 hive_parts: _,
                 predicate,
+                predicate_file_skip_applied: _,
                 scan_type,
                 unified_scan_args,
                 output_schema: _,
-                id: _,
             } => {
                 let name: &str = (&**scan_type).into();
                 let path = ScanSourcesDisplay(sources);
@@ -249,36 +262,37 @@ impl<'a> IRDotDisplay<'a> {
                 options,
                 ..
             } => {
-                self.with_root(*input_left)._format(f, Some(id), last)?;
-                self.with_root(*input_right)._format(f, Some(id), last)?;
-
-                let left_on = self.display_exprs(left_on);
-                let right_on = self.display_exprs(right_on);
+                recurse!(*input_left);
+                recurse!(*input_right);
 
                 write_label(f, id, |f| {
-                    write!(
-                        f,
-                        "JOIN {}\nleft: {left_on};\nright: {right_on}",
-                        options.args.how
-                    )
+                    write!(f, "JOIN {}", options.args.how)?;
+
+                    if !left_on.is_empty() {
+                        let left_on = self.display_exprs(left_on);
+                        let right_on = self.display_exprs(right_on);
+                        write!(f, "\nleft: {left_on};\nright: {right_on}")?
+                    }
+                    Ok(())
                 })?;
             },
             MapFunction {
                 input, function, ..
             } => {
-                self.with_root(*input)._format(f, Some(id), last)?;
+                recurse!(*input);
                 write_label(f, id, |f| write!(f, "{function}"))?;
             },
             ExtContext { input, .. } => {
-                self.with_root(*input)._format(f, Some(id), last)?;
+                recurse!(*input);
                 write_label(f, id, |f| f.write_str("EXTERNAL_CONTEXT"))?;
             },
             Sink { input, payload, .. } => {
-                self.with_root(*input)._format(f, Some(id), last)?;
+                recurse!(*input);
 
                 write_label(f, id, |f| {
                     f.write_str(match payload {
                         SinkTypeIR::Memory => "SINK (MEMORY)",
+                        SinkTypeIR::Callback { .. } => "SINK (CALLBACK)",
                         SinkTypeIR::File { .. } => "SINK (FILE)",
                         SinkTypeIR::Partition { .. } => "SINK (PARTITION)",
                     })
@@ -286,7 +300,7 @@ impl<'a> IRDotDisplay<'a> {
             },
             SinkMultiple { inputs } => {
                 for input in inputs {
-                    self.with_root(*input)._format(f, Some(id), last)?;
+                    recurse!(*input);
                 }
 
                 write_label(f, id, |f| f.write_str("SINK MULTIPLE"))?;
@@ -296,7 +310,7 @@ impl<'a> IRDotDisplay<'a> {
                 let total_columns = self.lp.lp_arena.get(*input).schema(self.lp.lp_arena).len();
 
                 let columns = ColumnsDisplay(columns.as_ref());
-                self.with_root(*input)._format(f, Some(id), last)?;
+                recurse!(*input);
                 write_label(f, id, |f| {
                     write!(f, "simple π {num_columns}/{total_columns}\n[{columns}]")
                 })?;
@@ -307,8 +321,8 @@ impl<'a> IRDotDisplay<'a> {
                 input_right,
                 key,
             } => {
-                self.with_root(*input_left)._format(f, Some(id), last)?;
-                self.with_root(*input_right)._format(f, Some(id), last)?;
+                recurse!(*input_left);
+                recurse!(*input_right);
 
                 write_label(f, id, |f| write!(f, "MERGE_SORTED ON '{key}'",))?;
             },
@@ -328,7 +342,7 @@ struct NumColumnsSchema<'a>(Option<&'a Schema>);
 impl fmt::Display for ScanSourceRef<'_> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
-            ScanSourceRef::Path(path) => path.display().fmt(f),
+            ScanSourceRef::Path(addr) => addr.display().fmt(f),
             ScanSourceRef::File(_) => f.write_str("open-file"),
             ScanSourceRef::Buffer(buff) => write!(f, "{} in-mem bytes", buff.len()),
         }
@@ -393,12 +407,11 @@ impl fmt::Write for EscapeLabel<'_> {
         loop {
             let mut char_indices = s.char_indices();
 
-            // This escapes quotes and new lines
-            // @NOTE: I am aware this does not work for \" and such. I am ignoring that fact as we
-            // are not really using such strings.
+            // This escapes quotes, new lines, and backslashes
             let f = char_indices.find_map(|(i, c)| match c {
                 '"' => Some((i, r#"\""#)),
                 '\n' => Some((i, r#"\n"#)),
+                '\\' => Some((i, r"\\")),
                 _ => None,
             });
 
@@ -419,10 +432,13 @@ impl fmt::Write for EscapeLabel<'_> {
 
 impl fmt::Display for IRDotDisplay<'_> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        writeln!(f, "graph  polars_query {{")?;
+        writeln!(f, "digraph polars_query {{")?;
+        writeln!(f, "{INDENT}rankdir=\"BT\"")?;
+        writeln!(f, "{INDENT}node [fontname=\"Monospace\", shape=\"box\"]")?;
 
         let mut last = 0;
-        self._format(f, None, &mut last)?;
+        let mut visited_caches = PlHashSet::new();
+        self._format(f, None, &mut last, &mut visited_caches)?;
 
         writeln!(f, "}}")?;
 
