@@ -105,22 +105,15 @@ impl OptimizationRule for SimpleProjectionAndCollapse {
                 }
             },
             // if there are 2 subsequent caches, flatten them and only take the inner
-            Cache {
-                input,
-                cache_hits: outer_cache_hits,
-                ..
-            } if !self.eager => {
+            Cache { input, .. } if !self.eager => {
                 if let Cache {
                     input: prev_input,
                     id,
-                    cache_hits,
                 } = lp_arena.get(*input)
                 {
                     Ok(Some(Cache {
                         input: *prev_input,
-                        id: id.clone(),
-                        // ensure the counts are updated
-                        cache_hits: cache_hits.saturating_add(*outer_cache_hits),
+                        id: *id,
                     }))
                 } else {
                     Ok(None)
