@@ -1,4 +1,4 @@
-use std::path::PathBuf;
+use std::num::NonZeroUsize;
 use std::sync::Arc;
 
 use polars_core::frame::DataFrame;
@@ -12,6 +12,7 @@ use polars_plan::dsl::deletion::DeletionFilesList;
 use polars_plan::dsl::{
     CastColumnsPolicy, JoinTypeOptionsIR, MissingColumnsPolicy, PartitionTargetCallback,
     PartitionVariantIR, ScanSources, SinkFinishCallback, SinkOptions, SinkTarget, SortColumnIR,
+    TableStatistics,
 };
 use polars_plan::plans::hive::HivePartitionsDf;
 use polars_plan::plans::{AExpr, DataFrameUdf, IR};
@@ -23,18 +24,23 @@ mod lower_expr;
 mod lower_group_by;
 mod lower_ir;
 mod to_graph;
+#[cfg(feature = "physical_plan_visualization")]
+pub mod visualization;
 
 pub use fmt::visualize_plan;
-use polars_plan::dsl::ExtraColumnsPolicy;
-use polars_plan::prelude::FileType;
+use polars_plan::prelude::{FileType, PlanCallback};
+use polars_time::{ClosedWindow, Duration};
 use polars_utils::arena::{Arena, Node};
 use polars_utils::pl_str::PlSmallStr;
+use polars_utils::plpath::PlPath;
 use polars_utils::slice_enum::Slice;
 use slotmap::{SecondaryMap, SlotMap};
 pub use to_graph::physical_plan_to_graph;
 
 pub use self::lower_ir::StreamingLowerIRContext;
-use crate::nodes::io_sources::multi_file_reader::reader_interface::builder::FileReaderBuilder;
+use crate::nodes::io_sources::multi_scan::components::forbid_extra_columns::ForbidExtraColumns;
+use crate::nodes::io_sources::multi_scan::components::projection::builder::ProjectionBuilder;
+use crate::nodes::io_sources::multi_scan::reader_interface::builder::FileReaderBuilder;
 use crate::physical_plan::lower_expr::ExprCache;
 
 slotmap::new_key_type! {
@@ -75,7 +81,7 @@ pub struct PhysStream {
 }
 
 impl PhysStream {
-    #[expect(unused)]
+    #[allow(unused)]
     pub fn new(node: PhysNodeKey, port: usize) -> Self {
         Self { node, port }
     }
@@ -87,6 +93,10 @@ impl PhysStream {
 }
 
 #[derive(Clone, Debug)]
+#[cfg_attr(
+    feature = "physical_plan_visualization",
+    derive(strum_macros::IntoStaticStr)
+)]
 pub enum PhysNodeKind {
     InMemorySource {
         df: Arc<DataFrame>,
@@ -98,16 +108,16 @@ pub enum PhysNodeKind {
         extend_original: bool,
     },
 
+    InputIndependentSelect {
+        selectors: Vec<ExprIR>,
+    },
+
     WithRowIndex {
         input: PhysStream,
         name: PlSmallStr,
         offset: Option<IdxSize>,
     },
 
-    InputIndependentSelect {
-        selectors: Vec<ExprIR>,
-    },
-
     Reduce {
         input: PhysStream,
         exprs: Vec<ExprIR>,
@@ -125,6 +135,18 @@ pub enum PhysNodeKind {
         length: usize,
     },
 
+    DynamicSlice {
+        input: PhysStream,
+        offset: PhysStream,
+        length: PhysStream,
+    },
+
+    Shift {
+        input: PhysStream,
+        offset: PhysStream,
+        fill: Option<PhysStream>,
+    },
+
     Filter {
         input: PhysStream,
         predicate: ExprIR,
@@ -139,6 +161,13 @@ pub enum PhysNodeKind {
         input: PhysStream,
     },
 
+    CallbackSink {
+        input: PhysStream,
+        function: PlanCallback<DataFrame, bool>,
+        maintain_order: bool,
+        chunk_size: Option<NonZeroUsize>,
+    },
+
     FileSink {
         target: SinkTarget,
         sink_options: SinkOptions,
@@ -149,7 +178,7 @@ pub enum PhysNodeKind {
 
     PartitionSink {
         input: PhysStream,
-        base_path: Arc<PathBuf>,
+        base_path: Arc<PlPath>,
         file_path_cb: Option<PartitionTargetCallback>,
         sink_options: SinkOptions,
         variant: PartitionVariantIR,
@@ -186,6 +215,38 @@ pub enum PhysNodeKind {
         sort_options: SortMultipleOptions,
     },
 
+    TopK {
+        input: PhysStream,
+        k: PhysStream,
+        by_column: Vec<ExprIR>,
+        reverse: Vec<bool>,
+        nulls_last: Vec<bool>,
+    },
+
+    Repeat {
+        value: PhysStream,
+        repeats: PhysStream,
+    },
+
+    #[cfg(feature = "cum_agg")]
+    CumAgg {
+        input: PhysStream,
+        kind: crate::nodes::cum_agg::CumAggKind,
+    },
+
+    // Parameter is the input stream
+    GatherEvery {
+        input: PhysStream,
+        n: usize,
+        offset: usize,
+    },
+    Rle(PhysStream),
+    RleId(PhysStream),
+    PeakMinMax {
+        input: PhysStream,
+        is_peak_max: bool,
+    },
+
     OrderedUnion {
         inputs: Vec<PhysStream>,
     },
@@ -210,21 +271,23 @@ pub enum PhysNodeKind {
         cloud_options: Option<Arc<CloudOptions>>,
 
         /// Columns to project from the file.
-        projected_file_schema: SchemaRef,
+        file_projection_builder: ProjectionBuilder,
         /// Final output schema of morsels being sent out of MultiScan.
         output_schema: SchemaRef,
 
         row_index: Option<RowIndex>,
         pre_slice: Option<Slice>,
         predicate: Option<ExprIR>,
+        predicate_file_skip_applied: Option<bool>,
 
         hive_parts: Option<HivePartitionsDf>,
         include_file_paths: Option<PlSmallStr>,
         cast_columns_policy: CastColumnsPolicy,
         missing_columns_policy: MissingColumnsPolicy,
-        extra_columns_policy: ExtraColumnsPolicy,
+        forbid_extra_columns: Option<ForbidExtraColumns>,
 
         deletion_files: Option<DeletionFilesList>,
+        table_statistics: Option<TableStatistics>,
 
         /// Schema of columns contained in the file. Does not contain external columns (e.g. hive / row_index).
         file_schema: SchemaRef,
@@ -242,6 +305,16 @@ pub enum PhysNodeKind {
         aggs: Vec<ExprIR>,
     },
 
+    #[cfg(feature = "dynamic_group_by")]
+    RollingGroupBy {
+        input: PhysStream,
+        index_column: PlSmallStr,
+        period: Duration,
+        offset: Duration,
+        closed: ClosedWindow,
+        aggs: Vec<ExprIR>,
+    },
+
     EquiJoin {
         input_left: PhysStream,
         input_right: PhysStream,
@@ -281,8 +354,24 @@ pub enum PhysNodeKind {
     MergeSorted {
         input_left: PhysStream,
         input_right: PhysStream,
+    },
 
-        key: PlSmallStr,
+    #[cfg(feature = "ewma")]
+    EwmMean {
+        input: PhysStream,
+        options: polars_ops::series::EWMOptions,
+    },
+
+    #[cfg(feature = "ewma")]
+    EwmVar {
+        input: PhysStream,
+        options: polars_ops::series::EWMOptions,
+    },
+
+    #[cfg(feature = "ewma")]
+    EwmStd {
+        input: PhysStream,
+        options: polars_ops::series::EWMOptions,
     },
 }
 
@@ -317,17 +406,34 @@ fn visit_node_inputs_mut(
             | PhysNodeKind::Filter { input, .. }
             | PhysNodeKind::SimpleProjection { input, .. }
             | PhysNodeKind::InMemorySink { input }
+            | PhysNodeKind::CallbackSink { input, .. }
             | PhysNodeKind::FileSink { input, .. }
             | PhysNodeKind::PartitionSink { input, .. }
             | PhysNodeKind::InMemoryMap { input, .. }
             | PhysNodeKind::Map { input, .. }
             | PhysNodeKind::Sort { input, .. }
             | PhysNodeKind::Multiplexer { input }
+            | PhysNodeKind::GatherEvery { input, .. }
+            | PhysNodeKind::Rle(input)
+            | PhysNodeKind::RleId(input)
+            | PhysNodeKind::PeakMinMax { input, .. }
             | PhysNodeKind::GroupBy { input, .. } => {
                 rec!(input.node);
                 visit(input);
             },
 
+            #[cfg(feature = "dynamic_group_by")]
+            PhysNodeKind::RollingGroupBy { input, .. } => {
+                rec!(input.node);
+                visit(input);
+            },
+
+            #[cfg(feature = "cum_agg")]
+            PhysNodeKind::CumAgg { input, .. } => {
+                rec!(input.node);
+                visit(input);
+            },
+
             PhysNodeKind::InMemoryJoin {
                 input_left,
                 input_right,
@@ -366,6 +472,50 @@ fn visit_node_inputs_mut(
                 visit(input_right);
             },
 
+            PhysNodeKind::TopK { input, k, .. } => {
+                rec!(input.node);
+                rec!(k.node);
+                visit(input);
+                visit(k);
+            },
+
+            PhysNodeKind::DynamicSlice {
+                input,
+                offset,
+                length,
+            } => {
+                rec!(input.node);
+                rec!(offset.node);
+                rec!(length.node);
+                visit(input);
+                visit(offset);
+                visit(length);
+            },
+
+            PhysNodeKind::Shift {
+                input,
+                offset,
+                fill,
+            } => {
+                rec!(input.node);
+                rec!(offset.node);
+                if let Some(fill) = fill {
+                    rec!(fill.node);
+                }
+                visit(input);
+                visit(offset);
+                if let Some(fill) = fill {
+                    visit(fill);
+                }
+            },
+
+            PhysNodeKind::Repeat { value, repeats } => {
+                rec!(value.node);
+                rec!(repeats.node);
+                visit(value);
+                visit(repeats);
+            },
+
             PhysNodeKind::OrderedUnion { inputs } | PhysNodeKind::Zip { inputs, .. } => {
                 for input in inputs {
                     rec!(input.node);
@@ -379,6 +529,14 @@ fn visit_node_inputs_mut(
                     visit(&mut PhysStream::first(*sink));
                 }
             },
+
+            #[cfg(feature = "ewma")]
+            PhysNodeKind::EwmMean { input, options: _ }
+            | PhysNodeKind::EwmVar { input, options: _ }
+            | PhysNodeKind::EwmStd { input, options: _ } => {
+                rec!(input.node);
+                visit(input)
+            },
         }
     }
 }
