@@ -4,7 +4,7 @@
 )]
 #![cfg_attr(feature = "nightly", feature(core_intrinsics))] // For algebraic ops, select_unpredictable.
 #![cfg_attr(feature = "nightly", allow(internal_features))]
-#![cfg_attr(docsrs, feature(doc_auto_cfg))]
+#![cfg_attr(docsrs, feature(doc_cfg))]
 pub mod abs_diff;
 pub mod algebraic_ops;
 pub mod arena;
@@ -15,7 +15,9 @@ pub mod cell;
 pub mod chunks;
 pub mod clmul;
 mod config;
+pub use config::check_allow_importing_interval_as_struct;
 pub mod cpuid;
+pub mod decimal;
 pub mod error;
 pub mod floor_divmod;
 pub mod functions;
@@ -25,10 +27,14 @@ pub mod idx_mapper;
 pub mod idx_vec;
 pub mod mem;
 pub mod min_max;
+pub mod order_statistic_tree;
 pub mod parma;
 pub mod pl_str;
+pub mod plpath;
 pub mod priority;
 pub mod regex_cache;
+pub mod relaxed_cell;
+pub mod row_counter;
 pub mod select;
 pub mod slice;
 pub mod slice_enum;
@@ -39,8 +45,10 @@ pub mod sync;
 pub mod sys;
 pub mod total_ord;
 pub mod unique_id;
+pub mod with_drop;
 
 pub use functions::*;
+pub mod compression;
 pub mod file;
 
 pub mod aliases;
@@ -48,9 +56,8 @@ pub mod fixedringbuffer;
 pub mod fmt;
 pub mod itertools;
 pub mod macros;
+pub mod option;
 pub mod vec;
-#[cfg(target_family = "wasm")]
-pub mod wasm;
 
 pub mod float;
 pub mod index;
@@ -74,3 +81,5 @@ pub mod python_convert_registry;
 pub mod pl_serialize;
 
 pub mod kahan_sum;
+pub use either;
+pub use idx_vec::UnitVec;
