@@ -5,7 +5,13 @@
 //! - Compact activated with `FC: false`
 use polars_error::{PolarsResult, to_compute_err};
 
-fn serialize_impl<W, T, const FC: bool>(writer: W, value: &T) -> PolarsResult<()>
+fn config() -> bincode::config::Configuration {
+    bincode::config::standard()
+        .with_no_limit()
+        .with_variable_int_encoding()
+}
+
+fn serialize_impl<W, T, const FC: bool>(mut writer: W, value: &T) -> PolarsResult<()>
 where
     W: std::io::Write,
     T: serde::ser::Serialize,
@@ -14,11 +20,13 @@ where
         let mut s = rmp_serde::Serializer::new(writer).with_struct_map();
         value.serialize(&mut s).map_err(to_compute_err)
     } else {
-        bincode::serialize_into(writer, value).map_err(to_compute_err)
+        bincode::serde::encode_into_std_write(value, &mut writer, config())
+            .map_err(to_compute_err)
+            .map(|_| ())
     }
 }
 
-pub fn deserialize_impl<T, R, const FC: bool>(reader: R) -> PolarsResult<T>
+pub fn deserialize_impl<T, R, const FC: bool>(mut reader: R) -> PolarsResult<T>
 where
     T: serde::de::DeserializeOwned,
     R: std::io::Read,
@@ -26,25 +34,10 @@ where
     if FC {
         rmp_serde::from_read(reader).map_err(to_compute_err)
     } else {
-        bincode::deserialize_from(reader).map_err(to_compute_err)
+        bincode::serde::decode_from_std_read(&mut reader, config()).map_err(to_compute_err)
     }
 }
 
-#[cfg(feature = "polars_cloud_server")]
-/// Deserializes the value and collects paths to all unknown fields.
-fn deserialize_with_unknown_fields<T, R>(reader: R) -> PolarsResult<(T, Vec<String>)>
-where
-    T: serde::de::DeserializeOwned,
-    R: std::io::Read,
-{
-    let mut de = rmp_serde::Deserializer::new(reader);
-    let mut unknown_fields = Vec::new();
-    let t = serde_ignored::deserialize(&mut de, |path| {
-        unknown_fields.push(path.to_string());
-    });
-    t.map(|t| (t, unknown_fields)).map_err(to_compute_err)
-}
-
 /// Mainly used to enable compression when serializing the final outer value.
 /// For intermediate serialization steps, the function in the module should
 /// be used instead.
@@ -87,25 +80,6 @@ impl SerializeOptions {
         }
     }
 
-    /// Deserializes the value and collects paths to all unknown fields.
-    ///
-    /// Supports only the future-compatible format (`FC: true`).
-    #[cfg(feature = "polars_cloud_server")]
-    pub fn deserialize_from_reader_with_unknown_fields<T, R>(
-        &self,
-        reader: R,
-    ) -> PolarsResult<(T, Vec<String>)>
-    where
-        T: serde::de::DeserializeOwned,
-        R: std::io::Read,
-    {
-        if self.compression {
-            deserialize_with_unknown_fields(flate2::read::ZlibDecoder::new(reader))
-        } else {
-            deserialize_with_unknown_fields(reader)
-        }
-    }
-
     pub fn serialize_to_bytes<T, const FC: bool>(&self, value: &T) -> PolarsResult<Vec<u8>>
     where
         T: serde::ser::Serialize,
@@ -152,6 +126,29 @@ where
     Ok(v)
 }
 
+/// Serialize function customized for `DslPlan`, with stack overflow protection.
+pub fn serialize_dsl<W, T>(writer: W, value: &T) -> PolarsResult<()>
+where
+    W: std::io::Write,
+    T: serde::ser::Serialize,
+{
+    let mut s = rmp_serde::Serializer::new(writer).with_struct_map();
+    let s = serde_stacker::Serializer::new(&mut s);
+    value.serialize(s).map_err(to_compute_err)
+}
+
+/// Deserialize function customized for `DslPlan`, with stack overflow protection.
+pub fn deserialize_dsl<T, R>(reader: R) -> PolarsResult<T>
+where
+    T: serde::de::DeserializeOwned,
+    R: std::io::Read,
+{
+    let mut de = rmp_serde::Deserializer::new(reader);
+    de.set_max_depth(usize::MAX);
+    let de = serde_stacker::Deserializer::new(&mut de);
+    T::deserialize(de).map_err(to_compute_err)
+}
+
 /// Potentially avoids copying memory compared to a naive `Vec::<u8>::deserialize`.
 ///
 /// This is essentially boilerplate for visiting bytes without copying where possible.
@@ -164,7 +161,7 @@ where
 {
     // Lets us avoid monomorphizing the visitor
     let mut out: Option<O> = None;
-    struct V<'f>(&'f mut (dyn for<'b> FnMut(std::borrow::Cow<'b, [u8]>)));
+    struct V<'f>(&'f mut dyn for<'b> FnMut(std::borrow::Cow<'b, [u8]>));
 
     deserializer.deserialize_bytes(V(&mut |v| drop(out.replace(func(v)))))?;
 
@@ -207,6 +204,81 @@ where
     }
 }
 
+thread_local! {
+    pub static USE_CLOUDPICKLE: std::cell::Cell<bool> = const { std::cell::Cell::new(false) };
+}
+
+#[cfg(feature = "python")]
+pub fn python_object_serialize(
+    pyobj: &pyo3::Py<pyo3::PyAny>,
+    buf: &mut Vec<u8>,
+) -> PolarsResult<()> {
+    use pyo3::Python;
+    use pyo3::pybacked::PyBackedBytes;
+    use pyo3::types::{PyAnyMethods, PyModule};
+
+    use crate::python_function::PYTHON3_VERSION;
+
+    let mut use_cloudpickle = USE_CLOUDPICKLE.get();
+    let dumped = Python::attach(|py| {
+        // Pickle with whatever pickling method was selected.
+        if use_cloudpickle {
+            let cloudpickle = PyModule::import(py, "cloudpickle")?.getattr("dumps")?;
+            cloudpickle.call1((pyobj.clone_ref(py),))?
+        } else {
+            let pickle = PyModule::import(py, "pickle")?.getattr("dumps")?;
+            match pickle.call1((pyobj.clone_ref(py),)) {
+                Ok(dumped) => dumped,
+                Err(_) => {
+                    use_cloudpickle = true;
+                    let cloudpickle = PyModule::import(py, "cloudpickle")?.getattr("dumps")?;
+                    cloudpickle.call1((pyobj.clone_ref(py),))?
+                },
+            }
+        }
+        .extract::<PyBackedBytes>()
+    })?;
+
+    // Write pickle metadata
+    buf.push(use_cloudpickle as u8);
+    buf.extend_from_slice(&*PYTHON3_VERSION);
+
+    // Write UDF
+    buf.extend_from_slice(&dumped);
+    Ok(())
+}
+
+#[cfg(feature = "python")]
+pub fn python_object_deserialize(buf: &[u8]) -> PolarsResult<pyo3::Py<pyo3::PyAny>> {
+    use polars_error::polars_ensure;
+    use pyo3::Python;
+    use pyo3::types::{PyAnyMethods, PyBytes, PyModule};
+
+    use crate::python_function::PYTHON3_VERSION;
+
+    // Handle pickle metadata
+    let use_cloudpickle = buf[0] != 0;
+    if use_cloudpickle {
+        let ser_py_version = &buf[1..3];
+        let cur_py_version = *PYTHON3_VERSION;
+        polars_ensure!(
+            ser_py_version == cur_py_version,
+            InvalidOperation:
+            "current Python version {:?} does not match the Python version used to serialize the UDF {:?}",
+            (3, cur_py_version[0], cur_py_version[1]),
+            (3, ser_py_version[0], ser_py_version[1] )
+        );
+    }
+    let buf = &buf[3..];
+
+    Python::attach(|py| {
+        let loads = PyModule::import(py, "pickle")?.getattr("loads")?;
+        let arg = (PyBytes::new(py, buf),);
+        let python_function = loads.call1(arg)?;
+        Ok(python_function.into())
+    })
+}
+
 #[cfg(test)]
 mod tests {
     #[test]
@@ -244,40 +316,4 @@ mod tests {
 
         assert_eq!(r, v);
     }
-
-    #[cfg(feature = "polars_cloud_server")]
-    #[test]
-    fn test_serde_collect_unknown_fields() {
-        #[derive(Clone, Copy, serde::Serialize)]
-        struct A {
-            x: bool,
-            u: u8,
-        }
-
-        #[derive(serde::Serialize)]
-        enum E {
-            V { val: A, ch: char },
-        }
-
-        #[derive(serde::Deserialize)]
-        struct B {
-            u: u8,
-        }
-
-        #[derive(serde::Deserialize)]
-        enum F {
-            V { val: B },
-        }
-
-        let a = A { u: 42, x: true };
-        let e = E::V { val: a, ch: 'x' };
-
-        let buf: Vec<u8> = super::serialize_to_bytes::<_, true>(&e).unwrap();
-        let (f, unknown) = super::deserialize_with_unknown_fields::<F, _>(buf.as_slice()).unwrap();
-
-        let F::V { val: b } = f;
-
-        assert_eq!(a.u, b.u);
-        assert_eq!(unknown.as_slice(), &["val.x", "ch"]);
-    }
 }
