@@ -3,7 +3,7 @@ use crate::{
     core_editor::Editor,
     menu_functions::{
         can_partially_complete, completer_input, floor_char_boundary, get_match_indices,
-        replace_in_buffer, style_suggestion,
+        replace_in_buffer, style_suggestion, truncate_no_ansi,
     },
     painting::Painter,
     Completer, Suggestion,
@@ -74,6 +74,8 @@ pub struct ColumnarMenu {
     working_details: ColumnDetails,
     /// Menu cached values
     values: Vec<Suggestion>,
+    /// Cached display width of each suggestion in `values`
+    display_widths: Vec<usize>,
     /// column position of the cursor. Starts from 0
     col_pos: u16,
     /// row position in the menu. Starts from 0
@@ -98,6 +100,7 @@ impl Default for ColumnarMenu {
             min_rows: 3,
             working_details: ColumnDetails::default(),
             values: Vec::new(),
+            display_widths: Vec::new(),
             col_pos: 0,
             row_pos: 0,
             skip_rows: 0,
@@ -381,7 +384,7 @@ impl ColumnarMenu {
         use_ansi_coloring: bool,
     ) -> String {
         let selected = index == self.index();
-        let empty_space = self.get_width().saturating_sub(suggestion.value.width());
+        let empty_space = self.get_width().saturating_sub(self.display_widths[index]);
 
         if use_ansi_coloring {
             // TODO(ysthakur): let the user strip quotes, rather than doing it here
@@ -394,53 +397,62 @@ impl ColumnarMenu {
             let match_indices =
                 get_match_indices(&suggestion.value, &suggestion.match_indices, shortest_base);
 
-            let left_text_size = self.longest_suggestion + self.default_details.col_padding;
+            let left_text_size = self
+                .get_width()
+                .min(self.longest_suggestion + self.default_details.col_padding);
             let description_size = self.get_width().saturating_sub(left_text_size);
-            let padding = left_text_size.saturating_sub(suggestion.value.len());
+            let padding = left_text_size.saturating_sub(self.display_widths[index]);
 
-            let value_style = if selected {
-                &self.settings.color.selected_text_style
+            let text_style = &suggestion.style.unwrap_or(self.settings.color.text_style);
+            let match_style = if selected {
+                &self.settings.color.selected_match_style
             } else {
-                &suggestion.style.unwrap_or(self.settings.color.text_style)
+                &self.settings.color.match_style
             };
+            let value_trunc = truncate_no_ansi(&suggestion.value, left_text_size);
             let styled_value = style_suggestion(
-                &value_style.paint(&suggestion.value).to_string(),
-                match_indices.as_ref(),
-                &self.settings.color.match_style,
+                &value_trunc,
+                &match_indices,
+                text_style,
+                match_style,
+                selected.then_some(&self.settings.color.selected_text_style),
             );
 
-            if let Some(description) = &suggestion.description {
-                let desc_trunc = description
-                    .chars()
-                    .take(description_size)
-                    .collect::<String>()
-                    .replace('\n', " ");
-                if selected {
+            match &suggestion.description {
+                Some(desc) if description_size > 3 => {
+                    let desc = desc.replace('\n', "");
+                    let desc_trunc = truncate_no_ansi(desc.as_str(), description_size);
+                    if selected {
+                        format!(
+                            "{}{}{}{}{}{}{}",
+                            styled_value,
+                            RESET,
+                            text_style.prefix(),
+                            self.settings.color.selected_text_style.prefix(),
+                            " ".repeat(padding),
+                            self.settings.color.description_style.paint(desc_trunc),
+                            RESET,
+                        )
+                    } else {
+                        format!(
+                            "{}{}{}{}{}",
+                            styled_value,
+                            RESET,
+                            " ".repeat(padding),
+                            self.settings.color.description_style.paint(desc_trunc),
+                            RESET,
+                        )
+                    }
+                }
+                _ => {
                     format!(
-                        "{}{}{}{}{}",
+                        "{}{}{:>empty$}",
                         styled_value,
-                        value_style.prefix(),
-                        " ".repeat(padding),
-                        desc_trunc,
-                        RESET,
-                    )
-                } else {
-                    format!(
-                        "{}{}{}{}",
-                        styled_value,
-                        " ".repeat(padding),
-                        self.settings.color.description_style.paint(desc_trunc),
                         RESET,
+                        "",
+                        empty = empty_space
                     )
                 }
-            } else {
-                format!(
-                    "{}{}{:>empty$}",
-                    styled_value,
-                    RESET,
-                    "",
-                    empty = empty_space
-                )
             }
         } else {
             // If no ansi coloring is found, then the selection word is the line in uppercase
@@ -538,6 +550,10 @@ impl Menu for ColumnarMenu {
 
     /// Updates menu values
     fn update_values(&mut self, editor: &mut Editor, completer: &mut dyn Completer) {
+        if self.settings.only_buffer_difference && self.input.is_none() {
+            self.input = Some(editor.get_buffer().to_string());
+        }
+
         let (input, pos) = completer_input(
             editor.get_buffer(),
             editor.insertion_point(),
@@ -548,6 +564,7 @@ impl Menu for ColumnarMenu {
         let (values, base_ranges) = completer.complete_with_base_ranges(&input, pos);
 
         self.values = values;
+        self.display_widths = self.values.iter().map(|sugg| sugg.value.width()).collect();
         self.working_details.shortest_base_string = base_ranges
             .iter()
             .map(|range| {
@@ -572,20 +589,13 @@ impl Menu for ColumnarMenu {
         if let Some(event) = self.event.take() {
             match event {
                 MenuEvent::Activate(updated) => {
-                    self.active = true;
                     self.reset_position();
 
-                    self.input = if self.settings.only_buffer_difference {
-                        Some(editor.get_buffer().to_string())
-                    } else {
-                        None
-                    };
-
                     if !updated {
                         self.update_values(editor, completer);
                     }
                 }
-                MenuEvent::Deactivate => self.active = false,
+                MenuEvent::Deactivate => {}
                 MenuEvent::Edit(updated) => {
                     self.reset_position();
 
@@ -614,43 +624,25 @@ impl Menu for ColumnarMenu {
                 .iter()
                 .any(|suggestion| suggestion.description.is_some());
 
+            let screen_width = painter.screen_width() as usize;
+            self.longest_suggestion = *self.display_widths.iter().max().unwrap_or(&0);
             if exist_description {
                 self.working_details.columns = 1;
-                self.working_details.col_width = painter.screen_width() as usize;
-
-                self.longest_suggestion = self.get_values().iter().fold(0, |prev, suggestion| {
-                    if prev >= suggestion.value.width() {
-                        prev
-                    } else {
-                        suggestion.value.width()
-                    }
-                });
+                self.working_details.col_width = screen_width;
             } else {
-                let max_width = self.get_values().iter().fold(0, |acc, suggestion| {
-                    let str_len = suggestion.value.width() + self.default_details.col_padding;
-                    if str_len > acc {
-                        str_len
-                    } else {
-                        acc
-                    }
-                });
-
                 // If no default width is found, then the total screen width is used to estimate
                 // the column width based on the default number of columns
                 let default_width = if let Some(col_width) = self.default_details.col_width {
                     col_width
                 } else {
-                    let col_width = painter.screen_width() / self.default_details.columns;
-                    col_width as usize
+                    screen_width / self.default_details.columns as usize
                 };
 
                 // Adjusting the working width of the column based the max line width found
                 // in the menu values
-                if max_width > default_width {
-                    self.working_details.col_width = max_width;
-                } else {
-                    self.working_details.col_width = default_width;
-                };
+                self.working_details.col_width = default_width
+                    .max(self.longest_suggestion + self.default_details.col_padding)
+                    .min(screen_width);
 
                 // The working columns is adjusted based on possible number of columns
                 // that could be fitted in the screen with the calculated column width
@@ -766,6 +758,8 @@ impl Menu for ColumnarMenu {
 
 #[cfg(test)]
 mod tests {
+    use std::io::BufWriter;
+
     use crate::{Span, UndoBehavior};
 
     use super::*;
@@ -873,6 +867,19 @@ mod tests {
         }
     }
 
+    fn setup_menu(
+        menu: &mut ColumnarMenu,
+        editor: &mut Editor,
+        completer: &mut dyn Completer,
+        terminal_size: (u16, u16),
+    ) {
+        let mut painter = Painter::new(BufWriter::new(std::io::stderr()));
+        painter.handle_resize(terminal_size.0, terminal_size.1);
+
+        menu.menu_event(MenuEvent::Activate(false));
+        menu.update_working_details(editor, completer, &painter);
+    }
+
     #[test]
     fn test_menu_replace_backtick() {
         // https://github.com/nushell/nushell/issues/7885
@@ -900,9 +907,9 @@ mod tests {
         let mut completer = FakeCompleter::new(&["おはよう", "`おはよう(`"]);
         let mut menu = ColumnarMenu::default().with_name("testmenu");
         let mut editor = Editor::default();
-
         editor.set_buffer("おは".to_string(), UndoBehavior::CreateUndoPoint);
-        menu.update_values(&mut editor, &mut completer);
+        setup_menu(&mut menu, &mut editor, &mut completer, (10, 10));
+
         assert!(menu.menu_string(2, true).contains("おは"));
     }
 
@@ -912,10 +919,10 @@ mod tests {
         let mut completer = FakeCompleter::new(&["验abc/"]);
         let mut menu = ColumnarMenu::default().with_name("testmenu");
         let mut editor = Editor::default();
-
         editor.set_buffer("ac".to_string(), UndoBehavior::CreateUndoPoint);
-        menu.update_values(&mut editor, &mut completer);
-        assert!(menu.menu_string(10, true).contains("验"));
+        setup_menu(&mut menu, &mut editor, &mut completer, (10, 10));
+
+        assert!(menu.menu_string(2, true).contains("验"));
     }
 
     #[test]
@@ -924,9 +931,9 @@ mod tests {
         let mut completer = FakeCompleter::new(&[&("验".repeat(205) + "abc/")]);
         let mut menu = ColumnarMenu::default().with_name("testmenu");
         let mut editor = Editor::default();
-
         editor.set_buffer("a".to_string(), UndoBehavior::CreateUndoPoint);
-        menu.update_values(&mut editor, &mut completer);
+        setup_menu(&mut menu, &mut editor, &mut completer, (10, 10));
+
         assert!(menu.menu_string(10, true).contains("验"));
     }
 
