@@ -8,6 +8,7 @@ use itertools::{
 };
 use nu_ansi_term::{ansi::RESET, Style};
 use unicode_segmentation::UnicodeSegmentation;
+use unicode_width::UnicodeWidthStr;
 
 use crate::{Editor, Suggestion, UndoBehavior};
 
@@ -186,7 +187,7 @@ pub fn find_common_string(values: &[Suggestion]) -> Option<(&Suggestion, usize)>
                 .char_indices()
                 .zip(current_suggestion.value.chars())
                 .find_map(|((idx, lhs), rhs)| (rhs != lhs).then_some(idx))
-                .unwrap_or(max_len);
+                .unwrap_or(current_suggestion.value.len());
             if new_common_prefix_len == 0 {
                 Done(0)
             } else {
@@ -416,7 +417,18 @@ fn parse_ansi(s: &str) -> Vec<(usize, usize, usize)> {
 ///
 /// * `match_indices` - Indices of graphemes (NOT bytes or chars) that matched the typed text
 /// * `match_style` - Style to use for matched characters
-pub fn style_suggestion(suggestion: &str, match_indices: &[usize], match_style: &Style) -> String {
+pub fn style_suggestion(
+    suggestion: &str,
+    match_indices: &[usize],
+    text_style: &Style,
+    match_style: &Style,
+    selected_style: Option<&Style>,
+) -> String {
+    let text_style_prefix = text_style.prefix().to_string();
+    let match_style_prefix = match_style.prefix().to_string();
+    let selected_prefix = selected_style
+        .map(|s| s.prefix().to_string())
+        .unwrap_or_default();
     let mut res = String::new();
     let mut offset = 0;
     for (escape_start, text_start, text_end) in parse_ansi(suggestion) {
@@ -425,14 +437,21 @@ pub fn style_suggestion(suggestion: &str, match_indices: &[usize], match_style:
         let graphemes = text.graphemes(true).collect::<Vec<_>>();
         let mut prev_matched = false;
 
+        res.push_str(&text_style_prefix);
+        res.push_str(&selected_prefix);
         res.push_str(escape);
         for (i, grapheme) in graphemes.iter().enumerate() {
             let is_match = match_indices.contains(&(i + offset));
 
             if is_match && !prev_matched {
-                res.push_str(&match_style.prefix().to_string());
+                res.push_str(RESET);
+                res.push_str(&text_style_prefix);
+                res.push_str(&match_style_prefix);
+                res.push_str(escape);
             } else if !is_match && prev_matched && i != 0 {
                 res.push_str(RESET);
+                res.push_str(&text_style_prefix);
+                res.push_str(&selected_prefix);
                 res.push_str(escape);
             }
             res.push_str(grapheme);
@@ -468,6 +487,31 @@ pub fn get_match_indices<'a>(
     }
 }
 
+/// Truncate a string with no ANSI escapes to the given max width, which must be >=3.
+///
+/// If `s` is longer than `max_width`, the resulting string will end in "..."
+/// and have width at most `max_width`.
+pub(crate) fn truncate_no_ansi(s: &str, max_width: usize) -> Cow<'_, str> {
+    if s.width() <= max_width {
+        Cow::Borrowed(s)
+    } else {
+        let trunc_suffix = "...";
+        let suffix_width = trunc_suffix.width();
+
+        let mut res = String::new();
+        let mut curr_width = 0;
+        for grapheme in s.graphemes(true) {
+            curr_width += grapheme.width();
+            if curr_width + suffix_width > max_width {
+                break;
+            }
+            res.push_str(grapheme);
+        }
+        res.push_str(trunc_suffix);
+        Cow::Owned(res)
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -720,6 +764,7 @@ mod tests {
     // https://github.com/nushell/nushell/pull/16765#issuecomment-3384411809
     #[case::unsorted(vec!["a", "b", "ab"], 0)]
     #[case::should_be_case_sensitive(vec!["a", "A"], 0)]
+    #[case::first_suggestion_longest(vec!["foobar", "foo"], 3)]
     fn test_find_common_string(#[case] input: Vec<&str>, #[case] expected: usize) {
         let input: Vec<_> = input
             .into_iter()
@@ -800,25 +845,52 @@ mod tests {
 
     #[test]
     fn style_fuzzy_suggestion() {
+        let text_style = Style::new().fg(Color::Red);
         let match_style = Style::new().underline();
+        let selected_style = Style::new().underline();
         let style1 = Style::new().on(Color::Blue);
         let style2 = Style::new().on(Color::Green);
 
         let expected = format!(
-            "{}{}{}{}{}{}{}{}{}{}{}{}{}",
+            "{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}{}",
+            text_style.prefix(),
+            selected_style.prefix(),
             style1.prefix(),
             "ab",
-            match_style.paint("汉"),
+            RESET,
+            text_style.prefix(),
+            match_style.prefix(),
+            style1.paint("汉"),
+            text_style.prefix(),
+            selected_style.prefix(),
             style1.prefix(),
             "d",
+            // TODO these next two are unnecessary, make sure consecutive ANSI escapes
+            // are treated as a single segment by parse_ansi()
+            text_style.prefix(),
+            selected_style.prefix(),
             RESET,
+            text_style.prefix(),
+            selected_style.prefix(),
             style2.prefix(),
-            match_style.paint("y̆👩🏾"),
+            RESET,
+            text_style.prefix(),
+            match_style.prefix(),
+            style2.paint("y̆👩🏾"),
+            text_style.prefix(),
+            selected_style.prefix(),
             style2.prefix(),
             "e",
+            text_style.prefix(),
+            selected_style.prefix(),
             RESET,
             "b@",
-            match_style.paint("r"),
+            RESET,
+            text_style.prefix(),
+            match_style.prefix(),
+            RESET,
+            "r",
+            RESET,
         );
         let match_indices = &[
             2, // 汉
@@ -830,7 +902,9 @@ mod tests {
             style_suggestion(
                 &format!("{}{}{}", style1.paint("ab汉d"), style2.paint("y̆👩🏾e"), "b@r"),
                 match_indices,
-                &match_style
+                &text_style,
+                &match_style,
+                Some(&selected_style),
             )
         );
     }
@@ -841,19 +915,26 @@ mod tests {
         let match_style = Style::new().underline();
 
         let expected = format!(
-            "{}{}{}{}",
+            "{}{}{}{}{}",
             text_style.prefix(),
             "go",
+            RESET,
+            text_style.prefix(),
             match_style.paint("o"),
-            RESET
         );
         assert_eq!(
             expected,
-            style_suggestion(
-                &text_style.paint("goo").to_string(),
-                &[2, 3, 4, 6],
-                &match_style
-            )
+            style_suggestion("goo", &[2, 3, 4, 6], &text_style, &match_style, None)
         );
     }
+
+    #[rstest]
+    #[case::shorter("asdf", 5, "asdf")]
+    // Ｈ has width 2
+    #[case::exact_width("asdＨ", 5, "asdＨ")]
+    #[case::one_longer("asdfＨ", 5, "as...")]
+    #[case::result_thinner_than_max("aＨＨＨ", 5, "a...")]
+    fn test_truncate(#[case] value: &str, #[case] max_width: usize, #[case] expected: &str) {
+        assert_eq!(expected, truncate_no_ansi(value, max_width));
+    }
 }
