@@ -2,6 +2,7 @@ use std::{borrow::Cow, sync::Arc};
 mod annotated;
 mod capabilities;
 mod content;
+mod elicitation_schema;
 mod extension;
 mod meta;
 mod prompt;
@@ -11,6 +12,7 @@ mod tool;
 pub use annotated::*;
 pub use capabilities::*;
 pub use content::*;
+pub use elicitation_schema::*;
 pub use extension::*;
 pub use meta::*;
 pub use prompt::*;
@@ -713,9 +715,9 @@ pub struct Icon {
     /// Optional override if the server's MIME type is missing or generic
     #[serde(skip_serializing_if = "Option::is_none")]
     pub mime_type: Option<String>,
-    /// Size specification (e.g., "48x48", "any" for SVG, or "48x48 96x96")
+    /// Size specification, each string should be in WxH format (e.g., `\"48x48\"`, `\"96x96\"`) or `\"any\"` for scalable formats like SVG
     #[serde(skip_serializing_if = "Option::is_none")]
-    pub sizes: Option<String>,
+    pub sizes: Option<Vec<String>>,
 }
 
 #[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
@@ -1377,7 +1379,21 @@ pub enum ElicitationAction {
 ///
 /// This structure contains everything needed to request interactive input from a user:
 /// - A human-readable message explaining what information is needed
-/// - A JSON schema defining the expected structure of the response
+/// - A type-safe schema defining the expected structure of the response
+///
+/// # Example
+///
+/// ```rust
+/// use rmcp::model::*;
+///
+/// let params = CreateElicitationRequestParam {
+///     message: "Please provide your email".to_string(),
+///     requested_schema: ElicitationSchema::builder()
+///         .required_email("email")
+///         .build()
+///         .unwrap(),
+/// };
+/// ```
 #[derive(Debug, Serialize, Deserialize, Clone, PartialEq)]
 #[serde(rename_all = "camelCase")]
 #[cfg_attr(feature = "schemars", derive(schemars::JsonSchema))]
@@ -1387,10 +1403,10 @@ pub struct CreateElicitationRequestParam {
     /// what information they need to provide.
     pub message: String,
 
-    /// JSON Schema defining the expected structure and validation rules for the user's response.
-    /// This allows clients to validate input and provide appropriate UI controls.
-    /// Must be a valid JSON Schema Draft 2020-12 object.
-    pub requested_schema: JsonObject,
+    /// Type-safe schema defining the expected structure and validation rules for the user's response.
+    /// This enforces the MCP 2025-06-18 specification that elicitation schemas must be objects
+    /// with primitive-typed properties.
+    pub requested_schema: ElicitationSchema,
 }
 
 /// The result returned by a client in response to an elicitation request.
@@ -2060,13 +2076,13 @@ mod tests {
         let icon = Icon {
             src: "https://example.com/icon.png".to_string(),
             mime_type: Some("image/png".to_string()),
-            sizes: Some("48x48".to_string()),
+            sizes: Some(vec!["48x48".to_string()]),
         };
 
         let json = serde_json::to_value(&icon).unwrap();
         assert_eq!(json["src"], "https://example.com/icon.png");
         assert_eq!(json["mimeType"], "image/png");
-        assert_eq!(json["sizes"], "48x48");
+        assert_eq!(json["sizes"][0], "48x48");
 
         // Test deserialization
         let deserialized: Icon = serde_json::from_value(json).unwrap();
@@ -2097,12 +2113,12 @@ mod tests {
                 Icon {
                     src: "https://example.com/icon.png".to_string(),
                     mime_type: Some("image/png".to_string()),
-                    sizes: Some("48x48".to_string()),
+                    sizes: Some(vec!["48x48".to_string()]),
                 },
                 Icon {
                     src: "https://example.com/icon.svg".to_string(),
                     mime_type: Some("image/svg+xml".to_string()),
-                    sizes: Some("any".to_string()),
+                    sizes: Some(vec!["any".to_string()]),
                 },
             ]),
             website_url: Some("https://example.com".to_string()),
@@ -2113,7 +2129,9 @@ mod tests {
         assert_eq!(json["websiteUrl"], "https://example.com");
         assert!(json["icons"].is_array());
         assert_eq!(json["icons"][0]["src"], "https://example.com/icon.png");
+        assert_eq!(json["icons"][0]["sizes"][0], "48x48");
         assert_eq!(json["icons"][1]["mimeType"], "image/svg+xml");
+        assert_eq!(json["icons"][1]["sizes"][0], "any");
     }
 
     #[test]
@@ -2143,7 +2161,7 @@ mod tests {
                 icons: Some(vec![Icon {
                     src: "https://example.com/server.png".to_string(),
                     mime_type: Some("image/png".to_string()),
-                    sizes: None,
+                    sizes: Some(vec!["48x48".to_string()]),
                 }]),
                 website_url: Some("https://docs.example.com".to_string()),
             },
@@ -2156,6 +2174,7 @@ mod tests {
             json["serverInfo"]["icons"][0]["src"],
             "https://example.com/server.png"
         );
+        assert_eq!(json["serverInfo"]["icons"][0]["sizes"][0], "48x48");
         assert_eq!(json["serverInfo"]["websiteUrl"], "https://docs.example.com");
     }
 }
