@@ -1,20 +1,19 @@
-use std::{
-    collections::HashMap,
-    sync::Arc,
-    time::{Duration, Instant},
-};
+use std::{collections::HashMap, sync::Arc, time::Duration};
 
 use oauth2::{
     AuthUrl, AuthorizationCode, ClientId, ClientSecret, CsrfToken, EmptyExtraTokenFields,
-    PkceCodeChallenge, PkceCodeVerifier, RedirectUrl, RefreshToken, Scope, StandardTokenResponse,
-    TokenResponse, TokenUrl,
+    PkceCodeChallenge, PkceCodeVerifier, RedirectUrl, RefreshToken, RequestTokenError, Scope,
+    StandardTokenResponse, TokenResponse, TokenUrl,
     basic::{BasicClient, BasicTokenType},
 };
-use reqwest::{Client as HttpClient, IntoUrl, StatusCode, Url, header::AUTHORIZATION};
+use reqwest::{
+    Client as HttpClient, IntoUrl, StatusCode, Url,
+    header::{AUTHORIZATION, WWW_AUTHENTICATE},
+};
 use serde::{Deserialize, Serialize};
 use thiserror::Error;
 use tokio::sync::{Mutex, RwLock};
-use tracing::{debug, error};
+use tracing::{debug, error, warn};
 
 const DEFAULT_EXCHANGE_URL: &str = "http://localhost";
 
@@ -102,7 +101,7 @@ pub enum AuthError {
 pub struct AuthorizationMetadata {
     pub authorization_endpoint: String,
     pub token_endpoint: String,
-    pub registration_endpoint: String,
+    pub registration_endpoint: Option<String>,
     pub issuer: Option<String>,
     pub jwks_uri: Option<String>,
     pub scopes_supported: Option<Vec<String>>,
@@ -111,6 +110,12 @@ pub struct AuthorizationMetadata {
     pub additional_fields: HashMap<String, serde_json::Value>,
 }
 
+#[derive(Debug, Clone, Deserialize)]
+struct ResourceServerMetadata {
+    authorization_server: Option<String>,
+    authorization_servers: Option<Vec<String>>,
+}
+
 /// oauth2 client config
 #[derive(Debug, Clone)]
 pub struct OAuthClientConfig {
@@ -148,7 +153,6 @@ pub struct AuthorizationManager {
     oauth_client: Option<OAuthClient>,
     credentials: RwLock<Option<OAuthTokenResponse>>,
     state: RwLock<Option<AuthorizationState>>,
-    expires_at: RwLock<Option<Instant>>,
     base_url: Url,
 }
 
@@ -165,7 +169,7 @@ pub struct ClientRegistrationRequest {
 pub struct ClientRegistrationResponse {
     pub client_id: String,
     pub client_secret: Option<String>,
-    pub client_name: String,
+    pub client_name: Option<String>,
     pub redirect_uris: Vec<String>,
     // allow additional fields
     #[serde(flatten)]
@@ -220,7 +224,6 @@ impl AuthorizationManager {
             oauth_client: None,
             credentials: RwLock::new(None),
             state: RwLock::new(None),
-            expires_at: RwLock::new(None),
             base_url,
         };
 
@@ -234,68 +237,17 @@ impl AuthorizationManager {
 
     /// discover oauth2 metadata
     pub async fn discover_metadata(&self) -> Result<AuthorizationMetadata, AuthError> {
-        for candidate_path in
-            Self::well_known_paths(self.base_url.path(), "oauth-authorization-server")
-        {
-            let mut discovery_url = self.base_url.clone();
-            discovery_url.set_path(&candidate_path);
-            debug!("discovery url: {:?}", discovery_url);
-
-            let response = match self
-                .http_client
-                .get(discovery_url)
-                .header("MCP-Protocol-Version", "2024-11-05")
-                .send()
-                .await
-            {
-                Ok(r) => r,
-                Err(e) => {
-                    debug!("discovery request failed: {}", e);
-                    continue; // try next candidate if request fails
-                }
-            };
-
-            if response.status() != StatusCode::OK {
-                debug!("discovery returned non-200: {}", response.status());
-                continue; // try next candidate if response is not OK
-            }
-
-            // parse metadata
-            let metadata = response
-                .json::<AuthorizationMetadata>()
-                .await
-                .map_err(|e| {
-                    // Fail the discovery if we get a 200 but cannot parse the response
-                    // This indicates a misconfiguration on the server side
-                    AuthError::MetadataError(format!("Failed to parse metadata: {}", e))
-                })?;
-            debug!("metadata: {:?}", metadata);
+        if let Some(metadata) = self.try_discover_oauth_server(&self.base_url).await? {
             return Ok(metadata);
         }
 
-        debug!("No valid .well-known endpoint found, falling back to default endpoints");
+        if let Some(metadata) = self.discover_oauth_server_via_resource_metadata().await? {
+            return Ok(metadata);
+        }
 
-        // fallback to default endpoints
-        let mut auth_base = self.base_url.clone();
-        // discard the path part, only keep scheme, host, port
-        auth_base.set_path("");
-
-        // Helper function to create endpoint URL
-        let create_endpoint = |path: &str| -> String {
-            let mut url = auth_base.clone();
-            url.set_path(path);
-            url.to_string()
-        };
-
-        Ok(AuthorizationMetadata {
-            authorization_endpoint: create_endpoint("authorize"),
-            token_endpoint: create_endpoint("token"),
-            registration_endpoint: create_endpoint("register"),
-            issuer: None,
-            jwks_uri: None,
-            scopes_supported: None,
-            additional_fields: HashMap::new(),
-        })
+        // No valid authorization metadata found - return error instead of guessing
+        // OAuth endpoints must be discovered from the server, not constructed by the client
+        Err(AuthError::NoAuthorizationSupport)
     }
 
     /// get client id and credentials
@@ -323,12 +275,10 @@ impl AuthorizationManager {
         let token_url = TokenUrl::new(metadata.token_endpoint.clone())
             .map_err(|e| AuthError::OAuthError(format!("Invalid token URL: {}", e)))?;
 
-        // debug!("token url: {:?}", token_url);
         let client_id = ClientId::new(config.client_id);
         let redirect_url = RedirectUrl::new(config.redirect_uri.clone())
             .map_err(|e| AuthError::OAuthError(format!("Invalid re URL: {}", e)))?;
 
-        debug!("client_id: {:?}", client_id);
         let mut client_builder = BasicClient::new(client_id.clone())
             .set_auth_uri(auth_url)
             .set_token_uri(token_url)
@@ -349,14 +299,16 @@ impl AuthorizationManager {
         redirect_uri: &str,
     ) -> Result<OAuthClientConfig, AuthError> {
         if self.metadata.is_none() {
-            error!("No authorization support detected");
             return Err(AuthError::NoAuthorizationSupport);
         }
 
         let metadata = self.metadata.as_ref().unwrap();
-        let registration_url = metadata.registration_endpoint.clone();
+        let Some(registration_url) = metadata.registration_endpoint.as_ref() else {
+            return Err(AuthError::RegistrationFailed(
+                "Dynamic client registration not supported".to_string(),
+            ));
+        };
 
-        debug!("registration url: {:?}", registration_url);
         // prepare registration request
         let registration_request = ClientRegistrationRequest {
             client_name: name.to_string(),
@@ -369,8 +321,6 @@ impl AuthorizationManager {
             response_types: vec!["code".to_string()],
         };
 
-        debug!("registration request: {:?}", registration_request);
-
         let response = match self
             .http_client
             .post(registration_url)
@@ -380,7 +330,6 @@ impl AuthorizationManager {
         {
             Ok(response) => response,
             Err(e) => {
-                error!("Registration request failed: {}", e);
                 return Err(AuthError::RegistrationFailed(format!(
                     "HTTP request error: {}",
                     e
@@ -395,7 +344,6 @@ impl AuthorizationManager {
                 Err(_) => "cannot get error details".to_string(),
             };
 
-            error!("Registration failed: HTTP {} - {}", status, error_text);
             return Err(AuthError::RegistrationFailed(format!(
                 "HTTP {}: {}",
                 status, error_text
@@ -406,7 +354,6 @@ impl AuthorizationManager {
         let reg_response = match response.json::<ClientRegistrationResponse>().await {
             Ok(response) => response,
             Err(e) => {
-                error!("Failed to parse registration response: {}", e);
                 return Err(AuthError::RegistrationFailed(format!(
                     "analyze response error: {}",
                     e
@@ -471,7 +418,6 @@ impl AuthorizationManager {
             pkce_verifier,
             csrf_token,
         });
-        debug!("set authorization state: {:?}", self.state.read().await);
 
         Ok(auth_url.to_string())
     }
@@ -507,19 +453,31 @@ impl AuthorizationManager {
         debug!("client_id: {:?}", oauth_client.client_id());
 
         // exchange token
-        let token_result = oauth_client
+        let token_result = match oauth_client
             .exchange_code(AuthorizationCode::new(code.to_string()))
             .set_pkce_verifier(pkce_verifier)
             .request_async(&http_client)
             .await
-            .map_err(|e| AuthError::TokenExchangeFailed(e.to_string()))?;
+        {
+            Ok(token) => token,
+            Err(RequestTokenError::Parse(_, body)) => {
+                match serde_json::from_slice::<OAuthTokenResponse>(&body) {
+                    Ok(parsed) => {
+                        warn!(
+                            "token exchange failed to parse completely but included a valid token response. Accepting it."
+                        );
+                        parsed
+                    }
+                    Err(parse_err) => {
+                        return Err(AuthError::TokenExchangeFailed(parse_err.to_string()));
+                    }
+                }
+            }
+            Err(e) => {
+                return Err(AuthError::TokenExchangeFailed(e.to_string()));
+            }
+        };
 
-        // get expires_in from token response
-        let expires_in = token_result.expires_in();
-        if let Some(expires_in) = expires_in {
-            let expires_at = Instant::now() + expires_in;
-            *self.expires_at.write().await = Some(expires_at);
-        }
         debug!("exchange token result: {:?}", token_result);
         // store credentials
         *self.credentials.write().await = Some(token_result.clone());
@@ -533,13 +491,15 @@ impl AuthorizationManager {
 
         if let Some(creds) = credentials.as_ref() {
             // check if the token is expire
-            if let Some(expires_at) = *self.expires_at.read().await {
-                if expires_at < Instant::now() {
-                    // token expired, try to refresh , release the lock
-                    drop(credentials);
-                    let new_creds = self.refresh_token().await?;
-                    return Ok(new_creds.access_token().secret().to_string());
-                }
+            let expires_in = creds.expires_in().unwrap_or(Duration::from_secs(0));
+            if expires_in <= Duration::from_secs(0) {
+                tracing::info!("Access token expired, refreshing.");
+                // token expired, try to refresh , release the lock
+                drop(credentials);
+
+                let new_creds = self.refresh_token().await?;
+                tracing::info!("Refreshed access token.");
+                return Ok(new_creds.access_token().secret().to_string());
             }
 
             Ok(creds.access_token().secret().to_string())
@@ -578,12 +538,6 @@ impl AuthorizationManager {
         // store new credentials
         *self.credentials.write().await = Some(token_result.clone());
 
-        // get expires_in from token response
-        let expires_in = token_result.expires_in();
-        if let Some(expires_in) = expires_in {
-            let expires_at = Instant::now() + expires_in;
-            *self.expires_at.write().await = Some(expires_at);
-        }
         Ok(token_result)
     }
 
@@ -608,6 +562,282 @@ impl AuthorizationManager {
             Ok(response)
         }
     }
+
+    async fn try_discover_oauth_server(
+        &self,
+        base_url: &Url,
+    ) -> Result<Option<AuthorizationMetadata>, AuthError> {
+        for candidate_path in Self::well_known_paths(base_url.path(), "oauth-authorization-server")
+        {
+            let mut discovery_url = base_url.clone();
+            discovery_url.set_query(None);
+            discovery_url.set_fragment(None);
+            discovery_url.set_path(&candidate_path);
+            if let Some(metadata) = self.fetch_authorization_metadata(&discovery_url).await? {
+                return Ok(Some(metadata));
+            }
+        }
+        Ok(None)
+    }
+
+    async fn fetch_authorization_metadata(
+        &self,
+        discovery_url: &Url,
+    ) -> Result<Option<AuthorizationMetadata>, AuthError> {
+        debug!("discovery url: {:?}", discovery_url);
+        let response = match self
+            .http_client
+            .get(discovery_url.clone())
+            .header("MCP-Protocol-Version", "2024-11-05")
+            .send()
+            .await
+        {
+            Ok(r) => r,
+            Err(e) => {
+                debug!("discovery request failed: {}", e);
+                return Ok(None);
+            }
+        };
+
+        if response.status() != StatusCode::OK {
+            debug!("discovery returned non-200: {}", response.status());
+            return Ok(None);
+        }
+
+        let metadata = response
+            .json::<AuthorizationMetadata>()
+            .await
+            .map_err(|e| AuthError::MetadataError(format!("Failed to parse metadata: {}", e)))?;
+        debug!("metadata: {:?}", metadata);
+        Ok(Some(metadata))
+    }
+
+    async fn discover_oauth_server_via_resource_metadata(
+        &self,
+    ) -> Result<Option<AuthorizationMetadata>, AuthError> {
+        let Some(resource_metadata_url) = self.discover_resource_metadata_url().await? else {
+            return Ok(None);
+        };
+
+        let Some(resource_metadata) = self
+            .fetch_resource_metadata_from_url(&resource_metadata_url)
+            .await?
+        else {
+            return Ok(None);
+        };
+
+        let mut candidates = Vec::new();
+
+        if let Some(single) = resource_metadata.authorization_server {
+            candidates.push(single);
+        }
+        if let Some(list) = resource_metadata.authorization_servers {
+            candidates.extend(list);
+        }
+
+        for candidate in candidates {
+            let candidate = candidate.trim();
+            if candidate.is_empty() {
+                continue;
+            }
+
+            let candidate_url = match Url::parse(candidate) {
+                Ok(url) => url,
+                Err(_) => match resource_metadata_url.join(candidate) {
+                    Ok(url) => url,
+                    Err(e) => {
+                        debug!("Failed to resolve authorization server URL `{candidate}`: {e}");
+                        continue;
+                    }
+                },
+            };
+
+            if candidate_url.path().contains("/.well-known/") {
+                if let Some(metadata) = self.fetch_authorization_metadata(&candidate_url).await? {
+                    return Ok(Some(metadata));
+                }
+                continue;
+            }
+
+            if let Some(metadata) = self.try_discover_oauth_server(&candidate_url).await? {
+                return Ok(Some(metadata));
+            }
+        }
+
+        Ok(None)
+    }
+
+    async fn discover_resource_metadata_url(&self) -> Result<Option<Url>, AuthError> {
+        if let Ok(Some(resource_metadata_url)) =
+            self.fetch_resource_metadata_url(&self.base_url).await
+        {
+            return Ok(Some(resource_metadata_url));
+        }
+
+        // If the primary URL doesn't use WWW-Authenticate, try oauth-protected-resource discovery.
+        // https://www.rfc-editor.org/rfc/rfc9728.html#name-obtaining-protected-resourc
+        for candidate_path in
+            Self::well_known_paths(self.base_url.path(), "oauth-protected-resource")
+        {
+            let mut discovery_url = self.base_url.clone();
+            discovery_url.set_query(None);
+            discovery_url.set_fragment(None);
+            discovery_url.set_path(&candidate_path);
+            if let Ok(Some(resource_metadata_url)) =
+                self.fetch_resource_metadata_url(&discovery_url).await
+            {
+                return Ok(Some(resource_metadata_url));
+            }
+        }
+
+        Ok(None)
+    }
+
+    /// Extract the resource metadata url from the WWW-Authenticate header value.
+    /// https://www.rfc-editor.org/rfc/rfc9728.html#name-use-of-www-authenticate-for
+    async fn fetch_resource_metadata_url(&self, url: &Url) -> Result<Option<Url>, AuthError> {
+        let response = match self
+            .http_client
+            .get(url.clone())
+            .header("MCP-Protocol-Version", "2024-11-05")
+            .send()
+            .await
+        {
+            Ok(r) => r,
+            Err(e) => {
+                debug!("resource metadata probe failed: {}", e);
+                return Ok(None);
+            }
+        };
+
+        if response.status() == StatusCode::OK {
+            return Ok(Some(url.clone()));
+        } else if response.status() != StatusCode::UNAUTHORIZED {
+            debug!(
+                "resource metadata probe returned unexpected status: {}",
+                response.status()
+            );
+            return Ok(None);
+        }
+
+        let mut parsed_url = None;
+        for value in response.headers().get_all(WWW_AUTHENTICATE).iter() {
+            let Ok(value_str) = value.to_str() else {
+                continue;
+            };
+            if let Some(url) =
+                Self::extract_resource_metadata_url_from_header(value_str, &self.base_url)
+            {
+                parsed_url = Some(url);
+                break;
+            }
+        }
+
+        Ok(parsed_url)
+    }
+
+    async fn fetch_resource_metadata_from_url(
+        &self,
+        resource_metadata_url: &Url,
+    ) -> Result<Option<ResourceServerMetadata>, AuthError> {
+        debug!(
+            "resource metadata discovery url: {:?}",
+            resource_metadata_url
+        );
+        let response = match self
+            .http_client
+            .get(resource_metadata_url.clone())
+            .header("MCP-Protocol-Version", "2024-11-05")
+            .send()
+            .await
+        {
+            Ok(r) => r,
+            Err(e) => {
+                debug!("resource metadata request failed: {}", e);
+                return Ok(None);
+            }
+        };
+
+        if response.status() != StatusCode::OK {
+            debug!(
+                "resource metadata request returned non-200: {}",
+                response.status()
+            );
+            return Ok(None);
+        }
+
+        let metadata = response
+            .json::<ResourceServerMetadata>()
+            .await
+            .map_err(|e| {
+                AuthError::MetadataError(format!("Failed to parse resource metadata: {}", e))
+            })?;
+        Ok(Some(metadata))
+    }
+
+    /// Extracts a url following `resource_metadata=` in a header value
+    fn extract_resource_metadata_url_from_header(header: &str, base_url: &Url) -> Option<Url> {
+        let header_lowercase = header.to_ascii_lowercase();
+        let fragment_key = "resource_metadata=";
+        let mut search_offset = 0;
+
+        while let Some(pos) = header_lowercase[search_offset..].find(fragment_key) {
+            let global_pos = search_offset + pos + fragment_key.len();
+            let value_slice = &header[global_pos..];
+            if let Some((value, consumed)) = Self::parse_next_header_value(value_slice) {
+                if let Ok(url) = Url::parse(&value) {
+                    return Some(url);
+                }
+                if let Ok(url) = base_url.join(&value) {
+                    return Some(url);
+                }
+                debug!("failed to parse resource metadata value `{value}` as URL");
+                search_offset = global_pos + consumed;
+                continue;
+            } else {
+                break;
+            }
+        }
+
+        None
+    }
+
+    /// Parses an authentication parameter value from a `WWW-Authenticate` header fragment.
+    /// The header fragment should start with the header value after the `=` character and then
+    /// reads until the value ends.
+    ///
+    /// Returns the extracted value together with the number of bytes consumed from the provided
+    /// fragment. Quoted values support escaped characters (e.g. `\"`). The parser skips leading
+    /// whitespace before reading either a quoted or token value. If no well-formed value is found,
+    /// `None` is returned.
+    fn parse_next_header_value(header_fragment: &str) -> Option<(String, usize)> {
+        let trimmed = header_fragment.trim_start();
+        let leading_ws = header_fragment.len() - trimmed.len();
+
+        if let Some(stripped) = trimmed.strip_prefix('"') {
+            let mut escaped = false;
+            let mut result = String::new();
+            #[allow(clippy::manual_strip)]
+            for (idx, ch) in stripped.char_indices() {
+                if escaped {
+                    result.push(ch);
+                    escaped = false;
+                    continue;
+                }
+                match ch {
+                    '\\' => escaped = true,
+                    '"' => return Some((result, leading_ws + idx + 2)),
+                    _ => result.push(ch),
+                }
+            }
+            None
+        } else {
+            let end = trimmed
+                .find(|c: char| c == ',' || c == ';' || c.is_whitespace())
+                .unwrap_or(trimmed.len());
+            Some((trimmed[..end].to_string(), leading_ws + end))
+        }
+    }
 }
 
 /// oauth2 authorization session, for guiding user to complete the authorization process
@@ -625,9 +855,9 @@ impl AuthorizationSession {
         redirect_uri: &str,
         client_name: Option<&str>,
     ) -> Result<Self, AuthError> {
-        // set redirect uri
+        // Default client config
         let config = OAuthClientConfig {
-            client_id: "mcp-client".to_string(), // temporary id, will be updated by dynamic registration
+            client_id: "mcp-client".to_string(),
             client_secret: None,
             scopes: scopes.iter().map(|s| s.to_string()).collect(),
             redirect_uri: redirect_uri.to_string(),
@@ -640,7 +870,10 @@ impl AuthorizationSession {
         {
             Ok(config) => config,
             Err(e) => {
-                eprintln!("Dynamic registration failed: {}", e);
+                warn!(
+                    "Dynamic registration failed: {}, fallback to default config",
+                    e
+                );
                 // fallback to default config
                 config
             }
@@ -924,8 +1157,78 @@ impl OAuthState {
 
 #[cfg(test)]
 mod tests {
+    use url::Url;
+
     use super::AuthorizationManager;
 
+    #[test]
+    fn parses_resource_metadata_parameter() {
+        let header = r#"Bearer error="invalid_request", error_description="missing token", resource_metadata="https://example.com/.well-known/oauth-protected-resource/api""#;
+        let base = Url::parse("https://example.com/api").unwrap();
+        let parsed = AuthorizationManager::extract_resource_metadata_url_from_header(header, &base);
+        assert_eq!(
+            parsed.unwrap().as_str(),
+            "https://example.com/.well-known/oauth-protected-resource/api"
+        );
+    }
+
+    #[test]
+    fn parses_relative_resource_metadata_parameter() {
+        let header = r#"Bearer error="invalid_request", resource_metadata="/.well-known/oauth-protected-resource/api""#;
+        let base = Url::parse("https://example.com/api").unwrap();
+        let parsed = AuthorizationManager::extract_resource_metadata_url_from_header(header, &base);
+        assert_eq!(
+            parsed.unwrap().as_str(),
+            "https://example.com/.well-known/oauth-protected-resource/api"
+        );
+    }
+
+    #[test]
+    fn parse_auth_param_value_handles_quoted_string() {
+        let fragment = r#""example", realm="foo""#;
+        let parsed = AuthorizationManager::parse_next_header_value(fragment).unwrap();
+        assert_eq!(parsed.0, "example");
+        assert_eq!(parsed.1, 9);
+    }
+
+    #[test]
+    fn parse_auth_param_value_handles_escaped_quotes_and_whitespace() {
+        let fragment = r#"   "a\"b\\c" ,next=value"#;
+        let parsed = AuthorizationManager::parse_next_header_value(fragment).unwrap();
+        assert_eq!(parsed.0, r#"a"b\c"#);
+        assert_eq!(parsed.1, 12);
+    }
+
+    #[test]
+    fn parse_auth_param_value_handles_token_values() {
+        let fragment = "  token,next";
+        let parsed = AuthorizationManager::parse_next_header_value(fragment).unwrap();
+        assert_eq!(parsed.0, "token");
+        assert_eq!(parsed.1, 7);
+    }
+
+    #[test]
+    fn parse_auth_param_value_handles_semicolon_separated_tokens() {
+        let fragment = r#"  https://example.com/meta; error="invalid_token""#;
+        let parsed = AuthorizationManager::parse_next_header_value(fragment).unwrap();
+        assert_eq!(parsed.0, "https://example.com/meta");
+        assert_eq!(&fragment[..parsed.1], "  https://example.com/meta");
+    }
+
+    #[test]
+    fn parse_auth_param_value_handles_semicolon_after_quoted_value() {
+        let fragment = r#"  "https://example.com/meta"; error="invalid_token""#;
+        let parsed = AuthorizationManager::parse_next_header_value(fragment).unwrap();
+        assert_eq!(parsed.0, "https://example.com/meta");
+        assert_eq!(&fragment[..parsed.1], r#"  "https://example.com/meta""#);
+    }
+
+    #[test]
+    fn parse_auth_param_value_returns_none_for_unterminated_quotes() {
+        let fragment = r#""unterminated,value"#;
+        assert!(AuthorizationManager::parse_next_header_value(fragment).is_none());
+    }
+
     #[test]
     fn well_known_paths_root() {
         let paths = AuthorizationManager::well_known_paths("/", "oauth-authorization-server");
