@@ -237,23 +237,22 @@ impl<R, D> TableOption<R, CompactMultilineConfig, D> for CompactMultilineConfig
 }
 
 mod print {
-    use std::{cmp::max, collections::HashMap, iter::repeat};
+    use std::{cmp::max, collections::HashMap, iter::repeat_n};
 
     use crate::{
         builder::Builder,
         grid::{
             ansi::ANSIStr,
             config::{
-                AlignmentHorizontal, AlignmentVertical, Border, Borders, ColoredConfig,
-                CompactMultilineConfig, Indent, Offset, Sides,
+                AlignmentHorizontal, AlignmentVertical, Border, Borders, CompactMultilineConfig,
+                Indent, Sides,
             },
-            dimension::{Dimension, DimensionPriority, Estimate, PoolTableDimension},
-            records::Records,
+            dimension::{DimensionPriority, PoolTableDimension},
             util::string::{
                 count_lines, get_line_width, get_lines, get_text_dimension, get_text_width,
             },
         },
-        settings::{Padding, Style, TableOption},
+        settings::{Padding, Style},
     };
 
     use super::TableValue;
@@ -644,9 +643,9 @@ mod print {
                 }
 
                 print_chars(&mut buf, pad.left.fill, pad_color.left, pad.left.size);
-                buf.extend(repeat(' ').take(left));
+                buf.extend(repeat_n(' ', left));
                 buf.push_str(&line);
-                buf.extend(repeat(' ').take(right));
+                buf.extend(repeat_n(' ', right));
                 print_chars(&mut buf, pad.right.fill, pad_color.right, pad.right.size);
 
                 if border.has_right() {
@@ -676,9 +675,9 @@ mod print {
                 }
 
                 print_chars(&mut buf, pad.left.fill, pad_color.left, pad.left.size);
-                buf.extend(repeat(' ').take(left));
+                buf.extend(repeat_n(' ', left));
                 buf.push_str(&line);
-                buf.extend(repeat(' ').take(right));
+                buf.extend(repeat_n(' ', right));
                 print_chars(&mut buf, pad.right.fill, pad_color.right, pad.right.size);
 
                 if border.has_right() {
@@ -732,10 +731,10 @@ mod print {
         match color {
             Some(color) => {
                 buf.push_str(color.get_prefix());
-                buf.extend(repeat(c).take(width));
+                buf.extend(repeat_n(c, width));
                 buf.push_str(color.get_suffix());
             }
-            None => buf.extend(repeat(c).take(width)),
+            None => buf.extend(repeat_n(c, width)),
         }
     }
 
@@ -814,7 +813,7 @@ mod print {
 
         let c = border.top.unwrap_or(' ');
         if splits.is_empty() {
-            buf.extend(repeat(c).take(width));
+            buf.extend(repeat_n(c, width));
         } else {
             let mut splits = splits;
             for i in 0..width {
@@ -892,7 +891,7 @@ mod print {
         }
 
         let c = border.bottom.unwrap_or(' ');
-        buf.extend(repeat(c).take(width));
+        buf.extend(repeat_n(c, width));
 
         if border.has_right() {
             if let Some(color) = color.right_bottom_corner {
@@ -983,104 +982,6 @@ mod print {
         }
     }
 
-    #[allow(dead_code)]
-    struct ConfigCell(PrintContext);
-
-    impl<R, D> TableOption<R, ColoredConfig, D> for ConfigCell {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            {
-                // we set a horizontal lines to borders to not complicate logic with cleaning it
-
-                let mut borders = *cfg.get_borders();
-                if let Some(line) = cfg.get_horizontal_line(0) {
-                    borders.top = line.main;
-                    borders.top_left = line.left;
-                    borders.top_right = line.right;
-                }
-
-                if let Some(line) = cfg.get_horizontal_line(1) {
-                    borders.bottom = line.main;
-                    borders.bottom_left = line.left;
-                    borders.bottom_right = line.right;
-                }
-
-                cfg_clear_borders(cfg);
-                cfg.set_borders(borders);
-            }
-
-            let mut ctx = self.0;
-
-            let has_vertical = cfg.get_borders().has_left();
-            if !ctx.intersections_horizontal.is_empty() && has_vertical {
-                let mut splits = short_splits(&mut ctx.intersections_horizontal, ctx.size.width);
-                squash_splits(&mut splits);
-
-                let c = cfg.get_borders().bottom_intersection.unwrap_or(' ');
-                cfg_set_top_chars(cfg, &splits, c)
-            }
-
-            let has_horizontal = cfg.get_borders().has_top();
-            if !ctx.intersections_vertical.is_empty() && has_horizontal {
-                let mut splits = short_splits(&mut ctx.intersections_vertical, ctx.size.height);
-                squash_splits(&mut splits);
-
-                let c = cfg.get_borders().right_intersection.unwrap_or(' ');
-                cfg_set_left_chars(cfg, &splits, c)
-            }
-
-            let mut borders = *cfg.get_borders();
-
-            // set top_left
-            {
-                if ctx.kv && ctx.kv_is_first {
-                    borders.top_left = borders.top_intersection;
-                }
-
-                if ctx.kv && !ctx.kv_is_first {
-                    borders.top_left = borders.intersection;
-                }
-
-                if ctx.kv && ctx.list && !ctx.list_is_first {
-                    borders.top_left = borders.left_intersection;
-                }
-
-                if ctx.is_first_col && !ctx.is_first_row {
-                    borders.top_left = borders.left_intersection;
-                }
-
-                if ctx.lean_top {
-                    borders.top_left = borders.top_intersection;
-                }
-
-                if ctx.top_left {
-                    borders.top_left = borders.left_intersection;
-                }
-
-                if ctx.top_intersection {
-                    borders.top_left = borders.intersection;
-                }
-            }
-
-            if ctx.is_last_col && !ctx.is_first_row {
-                borders.top_right = borders.right_intersection;
-            }
-
-            if !ctx.is_first_col && ctx.is_last_row {
-                borders.bottom_left = borders.bottom_intersection;
-            }
-
-            if !ctx.is_last_row || ctx.no_bottom {
-                cfg_no_bottom_borders(&mut borders);
-            }
-
-            if ctx.no_right {
-                cfg_no_right_borders(&mut borders);
-            }
-
-            cfg.set_borders(borders);
-        }
-    }
-
     fn cfg_no_bottom_borders(borders: &mut Borders<char>) {
         borders.bottom = None;
         borders.bottom_intersection = None;
@@ -1097,248 +998,6 @@ mod print {
         borders.vertical = None;
     }
 
-    #[allow(dead_code)]
-    fn cfg_set_top_chars(cfg: &mut ColoredConfig, list: &[usize], c: char) {
-        for &split in list {
-            let offset = split;
-            cfg.set_horizontal_char((0, 0), c, Offset::Begin(offset));
-        }
-    }
-
-    #[allow(dead_code)]
-    fn cfg_set_left_chars(cfg: &mut ColoredConfig, list: &[usize], c: char) {
-        for &offset in list {
-            cfg.set_vertical_char((0, 0), c, Offset::Begin(offset));
-        }
-    }
-
-    #[allow(dead_code)]
-    struct NoTopBorders;
-
-    impl<R, D> TableOption<R, ColoredConfig, D> for NoTopBorders {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.top = None;
-            borders.top_intersection = None;
-            borders.top_left = None;
-            borders.top_right = None;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    #[allow(dead_code)]
-    struct NoBottomBorders;
-
-    impl<R, D> TableOption<R, ColoredConfig, D> for NoBottomBorders {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.bottom = None;
-            borders.bottom_intersection = None;
-            borders.bottom_left = None;
-            borders.bottom_right = None;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    #[allow(dead_code)]
-    struct NoRightBorders;
-
-    impl<R, D> TableOption<R, ColoredConfig, D> for NoRightBorders {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.top_right = None;
-            borders.bottom_right = None;
-            borders.right = None;
-            borders.right_intersection = None;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    #[allow(dead_code)]
-    struct NoLeftBorders;
-
-    impl<R, D> TableOption<R, ColoredConfig, D> for NoLeftBorders {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.top_left = None;
-            borders.bottom_left = None;
-            borders.left = None;
-            borders.left_intersection = None;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    #[allow(dead_code)]
-    struct TopLeftChangeTopIntersection;
-
-    impl<R, D> TableOption<R, ColoredConfig, D> for TopLeftChangeTopIntersection {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.top_left = borders.top_intersection;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    #[allow(dead_code)]
-    struct TopLeftChangeIntersection;
-
-    impl<R, D> TableOption<R, ColoredConfig, D> for TopLeftChangeIntersection {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.top_left = borders.intersection;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    #[allow(dead_code)]
-    struct TopLeftChangeToLeft;
-
-    impl<R, D> TableOption<R, ColoredConfig, D> for TopLeftChangeToLeft {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.top_left = borders.left_intersection;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    #[allow(dead_code)]
-    struct TopRightChangeToRight;
-
-    impl<R, D> TableOption<R, ColoredConfig, D> for TopRightChangeToRight {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.top_right = borders.right_intersection;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    #[allow(dead_code)]
-    struct BottomLeftChangeSplit;
-
-    impl<R, D> TableOption<R, ColoredConfig, D> for BottomLeftChangeSplit {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.bottom_left = borders.left_intersection;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    #[allow(dead_code)]
-    struct BottomLeftChangeSplitToIntersection;
-
-    impl<R, D> TableOption<R, ColoredConfig, D> for BottomLeftChangeSplitToIntersection {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.bottom_left = borders.intersection;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    #[allow(dead_code)]
-    struct BottomRightChangeToRight;
-
-    impl<R, D> TableOption<R, ColoredConfig, D> for BottomRightChangeToRight {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.bottom_right = borders.right_intersection;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    #[allow(dead_code)]
-    struct BottomLeftChangeToBottomIntersection;
-
-    impl<R, D> TableOption<R, ColoredConfig, D> for BottomLeftChangeToBottomIntersection {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            let mut borders = *cfg.get_borders();
-            borders.bottom_left = borders.bottom_intersection;
-
-            cfg.set_borders(borders);
-        }
-    }
-
-    #[allow(dead_code)]
-    struct SetBottomChars<'a>(&'a [usize], char);
-
-    impl<R, D> TableOption<R, ColoredConfig, D> for SetBottomChars<'_>
-    where
-        R: Records,
-        for<'a> &'a R: Records,
-        for<'a> D: Dimension + Estimate<&'a R, ColoredConfig>,
-    {
-        fn change(self, records: &mut R, cfg: &mut ColoredConfig, dims: &mut D) {
-            dims.estimate(&*records, cfg);
-
-            let table_width = (0..records.count_columns())
-                .map(|col| dims.get_width(col))
-                .sum::<usize>()
-                + cfg.count_vertical(records.count_columns());
-            let mut current_width = 0;
-
-            for pos in self.0 {
-                current_width += pos;
-                if current_width > table_width {
-                    break;
-                }
-
-                let split_char = self.1;
-                cfg.set_horizontal_char((1, 0), split_char, Offset::Begin(current_width));
-
-                current_width += 1;
-            }
-        }
-    }
-
-    #[allow(dead_code)]
-    struct SetTopChars<'a>(&'a [usize], char);
-
-    impl<R, D> TableOption<R, ColoredConfig, D> for SetTopChars<'_> {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            for &split in self.0 {
-                let offset = split;
-                cfg.set_horizontal_char((0, 0), self.1, Offset::Begin(offset));
-            }
-        }
-    }
-
-    #[allow(dead_code)]
-    struct SetLeftChars<'a>(&'a [usize], char);
-
-    impl<R, D> TableOption<R, ColoredConfig, D> for SetLeftChars<'_> {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            for &offset in self.0 {
-                cfg.set_vertical_char((0, 0), self.1, Offset::Begin(offset));
-            }
-        }
-    }
-
-    struct GetTopIntersection(char);
-
-    impl<R, D> TableOption<R, ColoredConfig, D> for &mut GetTopIntersection {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            self.0 = cfg.get_borders().top_intersection.unwrap_or(' ');
-        }
-    }
-
-    struct GetBottomIntersection(char);
-
-    impl<R, D> TableOption<R, ColoredConfig, D> for &mut GetBottomIntersection {
-        fn change(self, _: &mut R, cfg: &mut ColoredConfig, _: &mut D) {
-            self.0 = cfg.get_borders().bottom_intersection.unwrap_or(' ');
-        }
-    }
-
     #[derive(Debug, Default)]
     struct Dimensions {
         all: HashMap<usize, Dim>,
@@ -1630,14 +1289,4 @@ mod print {
             (!pad_color.bottom.is_empty()).then_some(pad_color.bottom),
         )
     }
-
-    #[allow(dead_code)]
-    fn cfg_clear_borders(cfg: &mut ColoredConfig) {
-        cfg.remove_borders();
-        cfg.remove_borders_colors();
-        cfg.remove_vertical_chars();
-        cfg.remove_horizontal_chars();
-        cfg.remove_color_line_horizontal();
-        cfg.remove_color_line_vertical();
-    }
 }
